@@ -86,12 +86,13 @@ Key makeFsmKeyHex( char *str, const InputLoc &loc, Compiler *pd )
 
 	unsigned long ul = strtoul( str, 0, 16 );
 
+
 	if ( errno == ERANGE || (unusedBits && ul >> (size * 8)) ) {
 		error(loc) << "literal " << str << " overflows the alphabet type" << endl;
 		ul = 1 << (size * 8);
 	}
 
-	if ( unusedBits && ul >> (size * 8 - 1) )
+	if ( keyOps->alphType->isSigned && unusedBits && ul >> (size * 8 - 1) )
 		ul |= (ULONG_MAX >> (size*8 ) ) << (size*8);
 
 	return Key( (long)ul );
@@ -492,7 +493,8 @@ void Compiler::initGraphDict( )
 void Compiler::initKeyOps( )
 {
 	/* Signedness and bounds. */
-	HostType *alphType = alphTypeSet ? userAlphType : hostLang->defaultAlphType;
+	const HostType *alphType = alphTypeSet ? userAlphType :
+			&hostLang->hostTypes[hostLang->defaultHostType];
 	thisKeyOps.setAlphType( alphType );
 
 	if ( lowerNum != 0 ) {
@@ -760,10 +762,11 @@ FsmGraph *Compiler::makeScanner()
 }
 
 LangEl *Compiler::makeRepeatProd( const InputLoc &loc, Namespace *nspace,
-		const String &repeatName, UniqueType *ut )
+		const String &repeatName, UniqueType *ut, bool left )
 {
 	LangEl *prodName = addLangEl( this, nspace, repeatName, LangEl::NonTerm );
 	prodName->isRepeat = true;
+	prodName->leftRecursive = left;
 
 	ProdElList *prodElList1 = new ProdElList;
 
@@ -777,14 +780,20 @@ LangEl *Compiler::makeRepeatProd( const InputLoc &loc, Namespace *nspace,
 	ProdEl *factor2 = new ProdEl( ProdEl::ReferenceType,
 			InputLoc(), 0, false, typeRef2, 0 );
 
-	prodElList1->append( factor1 );
-	prodElList1->append( factor2 );
+	if ( left ) {
+		prodElList1->append( factor2 );
+		prodElList1->append( factor1 );
+	}
+	else {
+		prodElList1->append( factor1 );
+		prodElList1->append( factor2 );
+	}
 
 	Production *newDef1 = Production::cons( InputLoc(),
 			prodName, prodElList1, String(), false, 0,
-			prodList.length(), prodName->defList.length() );
+			prodList.length(), prodName->prodList.length() );
 
-	prodName->defList.append( newDef1 );
+	prodName->prodList.append( newDef1 );
 	prodList.append( newDef1 );
 
 	/* Build the second production of the repeat. */
@@ -792,19 +801,20 @@ LangEl *Compiler::makeRepeatProd( const InputLoc &loc, Namespace *nspace,
 
 	Production *newDef2 = Production::cons( InputLoc(),
 			prodName, prodElList2, String(), false, 0,
-			prodList.length(), prodName->defList.length() );
+			prodList.length(), prodName->prodList.length() );
 
-	prodName->defList.append( newDef2 );
+	prodName->prodList.append( newDef2 );
 	prodList.append( newDef2 );
 
 	return prodName;
 }
 
 LangEl *Compiler::makeListProd( const InputLoc &loc, Namespace *nspace,
-		const String &listName, UniqueType *ut )
+		const String &listName, UniqueType *ut, bool left )
 {
 	LangEl *prodName = addLangEl( this, nspace, listName, LangEl::NonTerm );
 	prodName->isList = true;
+	prodName->leftRecursive = left;
 
 	/* Build the first production of the list. */
 	TypeRef *typeRef1 = TypeRef::cons( loc, ut );
@@ -815,14 +825,20 @@ LangEl *Compiler::makeListProd( const InputLoc &loc, Namespace *nspace,
 	ProdEl *factor2 = new ProdEl( ProdEl::ReferenceType, loc, 0, false, typeRef2, 0 );
 
 	ProdElList *prodElList1 = new ProdElList;
-	prodElList1->append( factor1 );
-	prodElList1->append( factor2 );
+	if ( left ) {
+		prodElList1->append( factor2 );
+		prodElList1->append( factor1 );
+	}
+	else {
+		prodElList1->append( factor1 );
+		prodElList1->append( factor2 );
+	}
 
 	Production *newDef1 = Production::cons( loc,
 			prodName, prodElList1, String(), false, 0,
-			prodList.length(), prodName->defList.length() );
+			prodList.length(), prodName->prodList.length() );
 
-	prodName->defList.append( newDef1 );
+	prodName->prodList.append( newDef1 );
 	prodList.append( newDef1 );
 
 	/* Build the second production of the list. */
@@ -834,9 +850,9 @@ LangEl *Compiler::makeListProd( const InputLoc &loc, Namespace *nspace,
 
 	Production *newDef2 = Production::cons( loc,
 			prodName, prodElList2, String(), false, 0,
-			prodList.length(), prodName->defList.length() );
+			prodList.length(), prodName->prodList.length() );
 
-	prodName->defList.append( newDef2 );
+	prodName->prodList.append( newDef2 );
 	prodList.append( newDef2 );
 
 	return prodName;
@@ -857,9 +873,9 @@ LangEl *Compiler::makeOptProd( const InputLoc &loc, Namespace *nspace,
 
 	Production *newDef1 = Production::cons( loc,
 			prodName, prodElList1, String(), false, 0,
-			prodList.length(), prodName->defList.length() );
+			prodList.length(), prodName->prodList.length() );
 
-	prodName->defList.append( newDef1 );
+	prodName->prodList.append( newDef1 );
 	prodList.append( newDef1 );
 
 	/* Build the second production of the repeat. */
@@ -867,9 +883,9 @@ LangEl *Compiler::makeOptProd( const InputLoc &loc, Namespace *nspace,
 
 	Production *newDef2 = Production::cons( loc,
 			prodName, prodElList2, String(), false, 0,
-			prodList.length(), prodName->defList.length() );
+			prodList.length(), prodName->prodList.length() );
 
-	prodName->defList.append( newDef2 );
+	prodName->prodList.append( newDef2 );
 	prodList.append( newDef2 );
 
 	return prodName;
@@ -1008,7 +1024,7 @@ pda_run *Compiler::parsePattern( program_t *prg, tree_t **sp, const InputLoc &lo
 		if ( pdaRun->parse_error_text != 0 ) {
 			colm_data *tokdata = pdaRun->parse_error_text->tokdata;
 			cerr << ": relative error: ";
-			cerr.write( tokdata->data, tokdata->length );
+			cerr.write( (const char*)tokdata->data, tokdata->length );
 		}
 		else {
 			cerr << ": parse error";
