@@ -113,12 +113,12 @@ struct bindings
 	: public Vector<parse_tree_t*>
 {};
 
-struct DefListEl { Production *prev, *next; };
-struct LelDefListEl { Production *prev, *next; };
+struct ProdListEl { Production *prev, *next; };
+struct LelProdListEl { Production *prev, *next; };
 typedef Vector< LangEl* > LangElVect;
 typedef Vector< ProdEl* > FactorVect;
 
-typedef AvlMap<String, long, CmpStr> StringMap;
+typedef AvlMap<String, long, ColmCmpStr> StringMap;
 typedef AvlMapEl<String, long> StringMapEl;
 
 enum PredType { 
@@ -146,13 +146,13 @@ typedef DList<PredDecl> PredDeclList;
 /* Graph dictionary. */
 struct Production 
 :
-	public DefListEl, public LelDefListEl
+	public ProdListEl, public LelProdListEl
 {
 	Production()
 	: 
 		prodName(0), prodElList(0), prodCommit(false), redBlock(0),
 		prodId(0), prodNum(0), fsm(0), fsmLength(0), uniqueEmptyLeader(0),
-		isLeftRec(false), localFrame(0), lhsField(0), predOf(0)
+		isLeftRec(false), localFrame(0), lhsField(0), predOf(0), dotDotDot(false)
 	{}
 
 	static Production* cons( const InputLoc &loc, LangEl *prodName, ProdElList *prodElList, 
@@ -170,6 +170,14 @@ struct Production
 		return p;
 	}
 
+	static Production *cons( const InputLoc &loc )
+	{
+		Production *p = new Production;
+		p->loc = loc;
+		p->dotDotDot = true;
+		return p;
+	}
+
 	InputLoc loc;
 	LangEl *prodName;
 	ProdElList *prodElList;
@@ -198,6 +206,7 @@ struct Production
 	LangEl *predOf;
 
 	UnsignedCharVect copy;
+	bool dotDotDot;
 };
 
 struct CmpDefById
@@ -218,15 +227,15 @@ struct CmpDefById
 typedef BstMap< int, Production*, CmpOrd<int> > DotItemIndex;
 typedef BstMapEl< int, Production*> DotItemIndexEl;
 
-struct DefList
+struct ProdList
 :
-	public DListMel<Production, DefListEl>
+	public DListMel<Production, ProdListEl>
 {};
 
 /* A vector of production vectors. Each non terminal can have many productions. */
-struct LelDefList
+struct LelProdList
 :
-	public DListMel<Production, LelDefListEl> 
+	public DListMel<Production, LelProdListEl> 
 {};
 
 /* A set of machines made during a closure round. */
@@ -281,10 +290,13 @@ struct LangEl : public DListEl<LangEl>
 	bool parseStop;
 	bool isEOF;
 
+	/* For a list or a repeat. Defaults to right recursive. */
+	bool leftRecursive;
+
 	LangEl *repeatOf;
 
 	/* Productions from the language element if it is a non-terminal. */
-	LelDefList defList;
+	LelProdList prodList;
 
 	TokenDef *tokenDef;
 	Production *rootDef;
@@ -488,7 +500,7 @@ struct TokenRegion;
 
 /* tree_t of instantiated names. */
 typedef BstMapEl<String, NameInst*> NameMapEl;
-typedef BstMap<String, NameInst*, CmpStr> NameMap;
+typedef BstMap<String, NameInst*, ColmCmpStr> NameMap;
 typedef Vector<NameInst*> NameVect;
 typedef BstSet<NameInst*> NameSet;
 
@@ -593,7 +605,7 @@ struct Compiler
 	int nextPriorKey, nextNameId;
 
 	/* Alphabet type. */
-	HostType *userAlphType;
+	const HostType *userAlphType;
 	bool alphTypeSet;
 
 	/* Element type and get key expression. */
@@ -739,9 +751,9 @@ struct Compiler
 	void printFirstSets();
 
 	LangEl *makeRepeatProd( const InputLoc &loc, Namespace *nspace,
-			const String &repeatName, UniqueType *ut );
+			const String &repeatName, UniqueType *ut, bool left );
 	LangEl *makeListProd( const InputLoc &loc, Namespace *nspace,
-			const String &listName, UniqueType *ut );
+			const String &listName, UniqueType *ut, bool left );
 	LangEl *makeOptProd( const InputLoc &loc, Namespace *nspace,
 			const String &optName, UniqueType *ut );
 	void resolveProdEl( ProdEl *prodEl );
@@ -873,7 +885,7 @@ struct Compiler
 
 	LelList langEls;
 	StructElList structEls;
-	DefList prodList;
+	ProdList prodList;
 
 	/* Dumping. */
 	DotItemIndex dotItemIndex;
@@ -1104,7 +1116,7 @@ void errorStateLabels( const NameSet &locations );
 
 struct ColmParser;
 
-typedef AvlMap<String, ColmParser *, CmpStr> ParserDict;
+typedef AvlMap<String, ColmParser *, ColmCmpStr> ParserDict;
 typedef AvlMapEl<String, ColmParser *> ParserDictEl;
 
 LangEl *declareLangEl( Compiler *pd, Namespace *nspace,
