@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2015 Adrian Thurston <thurston@colm.net>
+ * Copyright 2006-2018 Adrian Thurston <thurston@colm.net>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -343,21 +343,21 @@ ProdEl *ConsInit::prodRefName( const String &capture, const String &name )
 	return prodEl;
 }
 
-ProdEl *ConsInit::prodRefNameRepeat( const String &name )
+ProdEl *ConsInit::prodRefNameLeftRepeat( const String &name )
 {
 	ProdEl *prodEl = prodElName( internal, name,
 			NamespaceQual::cons( curNspace() ), 0,
-			RepeatRepeat, false );
+			RepeatLeftRepeat, false );
 	return prodEl;
 }
 
-ProdEl *ConsInit::prodRefNameRepeat( const String &capture, const String &name )
+ProdEl *ConsInit::prodRefNameLeftRepeat( const String &capture, const String &name )
 {
 	ObjectField *captureField = ObjectField::cons( internal,
 			ObjectField::RhsNameType, 0, capture );
 	ProdEl *prodEl = prodElName( internal, name,
 			NamespaceQual::cons( curNspace() ), captureField,
-			RepeatRepeat, false );
+			RepeatLeftRepeat, false );
 	return prodEl;
 }
 
@@ -441,53 +441,53 @@ Production *ConsInit::production( ProdEl *prodEl1, ProdEl *prodEl2,
 void ConsInit::definition( const String &name, Production *prod1, Production *prod2,
 		Production *prod3, Production *prod4 )
 {
-	LelDefList *defList = new LelDefList;
-	prodAppend( defList, prod1 );
-	prodAppend( defList, prod2 );
-	prodAppend( defList, prod3 );
-	prodAppend( defList, prod4 );
+	LelProdList *prodList = new LelProdList;
+	prodAppend( prodList, prod1 );
+	prodAppend( prodList, prod2 );
+	prodAppend( prodList, prod3 );
+	prodAppend( prodList, prod4 );
 
 	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
 	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
 			name, pd->nextObjectId++ ); 
-	cflDef( ntDef, objectDef, defList );
+	cflDef( ntDef, objectDef, prodList );
 }
 
 void ConsInit::definition( const String &name, Production *prod1,
 		Production *prod2, Production *prod3 )
 {
-	LelDefList *defList = new LelDefList;
-	prodAppend( defList, prod1 );
-	prodAppend( defList, prod2 );
-	prodAppend( defList, prod3 );
+	LelProdList *prodList = new LelProdList;
+	prodAppend( prodList, prod1 );
+	prodAppend( prodList, prod2 );
+	prodAppend( prodList, prod3 );
 
 	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
 	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
 			name, pd->nextObjectId++ ); 
-	cflDef( ntDef, objectDef, defList );
+	cflDef( ntDef, objectDef, prodList );
 }
 
 void ConsInit::definition( const String &name, Production *prod1, Production *prod2 )
 {
-	LelDefList *defList = new LelDefList;
-	prodAppend( defList, prod1 );
-	prodAppend( defList, prod2 );
+	LelProdList *prodList = new LelProdList;
+	prodAppend( prodList, prod1 );
+	prodAppend( prodList, prod2 );
 
 	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
 	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
 			name, pd->nextObjectId++ ); 
-	cflDef( ntDef, objectDef, defList );
+	cflDef( ntDef, objectDef, prodList );
 }
 
 void ConsInit::definition( const String &name, Production *prod )
 {
-	LelDefList *defList = new LelDefList;
-	prodAppend( defList, prod );
+	LelProdList *prodList = new LelProdList;
+	prodAppend( prodList, prod );
 
 	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
 	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
 			name, pd->nextObjectId++ ); 
-	cflDef( ntDef, objectDef, defList );
+	cflDef( ntDef, objectDef, prodList );
 }
 
 void ConsInit::lexFactor()
@@ -631,8 +631,9 @@ void ConsInit::optProdElName()
 
 void ConsInit::optNi()
 {
-	ProdEl *prodEl1 = prodRefName( "Ni", "NI" );
-	Production *prod1 = production( prodEl1 );
+	ProdEl *prodEl1 = prodRefLit( "'-'" );
+	ProdEl *prodEl2 = prodRefName( "Ni", "NI" );
+	Production *prod1 = production( prodEl1, prodEl2 );
 
 	Production *prod2 = production();
 
@@ -644,9 +645,12 @@ void ConsInit::optRepeat()
 	ProdEl *prodEl1 = prodRefName( "Star", "STAR" );
 	Production *prod1 = production( prodEl1 );
 
-	Production *prod2 = production();
+	ProdEl *prodEl2 = prodRefName( "LeftStar", "LEFT_STAR" );
+	Production *prod2 = production( prodEl2 );
 
-	definition( "opt_prod_repeat",  prod1, prod2 );
+	Production *prod3 = production();
+
+	definition( "opt_prod_repeat",  prod1, prod2, prod3 );
 }
 
 void ConsInit::prodEl()
@@ -691,6 +695,30 @@ void ConsInit::optProdName()
 	definition( "opt_prod_name",  prod1, prod2 );
 }
 
+void ConsInit::prodVarDef()
+{
+	ProdEl *prodEl1 = prodRefName( "Name", "id" );
+	ProdEl *prodEl2 = prodRefLit( "':'" );
+	ProdEl *prodEl3 = prodRefName( "Type", "id" );
+	Production *prod1 = production( prodEl1, prodEl2, prodEl3 );
+
+	definition( "prod_var_def", prod1 );
+}
+
+/* The prod var list we provide in a basic form, just "id: id." We won't make
+ * use of them in bootstrap0 or bootstrap1, They are ignored in the loader for
+ * bootstrap1. We want to use them in bootstrap2 during the rewrite stage. */
+void ConsInit::prodVarList()
+{
+	ProdEl *prodEl1 = prodRefName( "VarDefList", "prod_var_list" );
+	ProdEl *prodEl2 = prodRefName( "VarDef", "prod_var_def" );
+	Production *prod1 = production( prodEl1, prodEl2 );
+
+	Production *prod2 = production();
+
+	definition( "prod_var_list",  prod1, prod2 );
+}
+
 void ConsInit::prod()
 {
 	ProdEl *prodEl1 = prodRefLit( "'['" );
@@ -720,9 +748,10 @@ Production *ConsInit::prodProd()
 {
 	ProdEl *prodEl1 = prodRefLit( "'def'" );
 	ProdEl *prodEl2 = prodRefName( "DefId", "id" );
-	ProdEl *prodEl3 = prodRefName( "ProdList", "prod_list" );
+	ProdEl *prodEl3 = prodRefName( "ProdVarList", "prod_var_list" );
+	ProdEl *prodEl4 = prodRefName( "ProdList", "prod_list" );
 
-	return production( prodEl1, prodEl2, prodEl3 );
+	return production( prodEl1, prodEl2, prodEl3, prodEl4 );
 }
 
 void ConsInit::item()
@@ -734,7 +763,7 @@ void ConsInit::item()
 
 void ConsInit::startProd()
 {
-	ProdEl *prodEl1 = prodRefNameRepeat( "ItemList", "item" );
+	ProdEl *prodEl1 = prodRefNameLeftRepeat( "ItemList", "item" );
 	Production *prod1 = production( prodEl1 );
 
 	definition( "start",  prod1 );
@@ -785,7 +814,7 @@ void ConsInit::parseInput( StmtList *stmtList )
 	LangExpr *open = LangExpr::cons( LangTerm::cons( InputLoc(), openRef, openArgs ) );
 
 	/* Construct a list containing the open stream. */
-	ConsItem *consItem = ConsItem::cons( internal, ConsItem::ExprType, open, false );
+	ConsItem *consItem = ConsItem::cons( internal, ConsItem::ExprType, open, ConsItem::TrimDefault );
 	ConsItemList *list = ConsItemList::cons( consItem );
 
 	/* Will capture the parser to "P" */
@@ -799,7 +828,7 @@ void ConsInit::parseInput( StmtList *stmtList )
 
 	/* Parse the above list. */
 	LangExpr *parseExpr = parseCmd( internal, false, false, objField,
-			typeRef, 0, list, true, false, "" );
+			typeRef, 0, list, true, false, false, "" );
 	LangStmt *parseStmt = LangStmt::cons( internal, LangStmt::ExprType, parseExpr );
 	stmtList->append( parseStmt );
 }
@@ -847,6 +876,7 @@ void ConsInit::go( long activeRealm )
 
 	keyword( "STAR", "'*'");
 	keyword( "PLUS", "'+'");
+	keyword( "LEFT_STAR", "'<*'");
 	keyword( "'['" );
 	keyword( "']'" );
 	keyword( "'|'" );
@@ -858,6 +888,7 @@ void ConsInit::go( long activeRealm )
 	keyword( "')'" );
 	keyword( "'..'" );
 	keyword( "'^'" );
+	keyword( "'-'" );
 
 	popRegionSet();
 
@@ -874,6 +905,8 @@ void ConsInit::go( long activeRealm )
 	prodElList();
 	optCommit();
 	optProdName();
+	prodVarDef();
+	prodVarList();
 	prod();
 	prodList();
 	ignore();
