@@ -1,5 +1,5 @@
 /*
- * Copyright 2002 Adrian Thurston <thurston@colm.net>
+ * Copyright 2002-2018 Adrian Thurston <thurston@colm.net>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -852,7 +852,7 @@ template<class T> struct CmpStrTmpl
 	}
 };
 
-typedef CmpStrTmpl<char> CmpStr;
+typedef CmpStrTmpl<char> ColmCmpStr;
 
 #ifdef AAPL_NAMESPACE
 }
