@@ -50,6 +50,7 @@ void Compiler::initUniqueTypes( )
 	uniqeTypeMap.insert( uniqueTypeIgnore );
 	uniqeTypeMap.insert( uniqueTypeAny );
 
+	uniqeTypeMap.insert( uniqueTypeInput );
 	uniqeTypeMap.insert( uniqueTypeStream );
 }
 
@@ -252,8 +253,8 @@ void Compiler::declareReVars()
 LangEl *declareLangEl( Compiler *pd, Namespace *nspace,
 		const String &data, LangEl::Type type )
 {
-    /* If the id is already in the dict, it will be placed in last found. If
-     * it is not there then it will be inserted and last found will be set to it. */
+	/* If the id is already in the dict, it will be placed in last found. If it
+	 * is not there then it will be inserted and last found will be set to it. */
 	TypeMapEl *inDict = nspace->typeMap.find( data );
 	if ( inDict != 0 )
 		error() << "language element '" << data << "' already defined as something else" << endp;
@@ -300,8 +301,8 @@ LangEl *addLangEl( Compiler *pd, Namespace *inNspace,
 void declareTypeAlias( Compiler *pd, Namespace *nspace,
 		const String &data, TypeRef *typeRef )
 {
-    /* If the id is already in the dict, it will be placed in last found. If
-     * it is not there then it will be inserted and last found will be set to it. */
+	/* If the id is already in the dict, it will be placed in last found. If
+	 * it is not there then it will be inserted and last found will be set to it. */
 	TypeMapEl *inDict = nspace->typeMap.find( data );
 	if ( inDict != 0 )
 		error() << "alias '" << data << "' already defined as something else" << endp;
@@ -470,16 +471,48 @@ void Namespace::declare( Compiler *pd )
 
 	for ( NtDefList::Iter n = ntDefList; n.lte(); n++ ) {
 		/* Get the language element. */
+		if ( n->isRedef ) {
+			TypeMapEl *inDict = this->typeMap.find( n->name );
+			if ( inDict == 0 ) {
+				error() << "redef not found '" << n->name << endp;
+			}
+
+			LangEl *langEl = inDict->value;
+			this->typeMap.remove( n->name );
+
+			/* Traverse the list of productions, moving to combined and taking
+			 * the original language element's productions when we enounter
+			 * [...]. Afterwards move the newly combined productions
+			 * to the nonterminal-definition and proceed with the declaration. */
+			LelProdList combined;
+			while ( n->prodList->length() > 0 ) {
+				Production *prod = n->prodList->detachFirst();
+				if ( prod->dotDotDot ) {
+					while ( langEl->prodList.length() ) {
+						Production *prod = langEl->prodList.detachFirst();
+						prod->prodNum = combined.length();
+						combined.append( prod );
+					}
+				}
+				else {
+					prod->prodNum = combined.length();
+					combined.append( prod );
+				}
+			}
+
+			/* Move bak to non-term def and proceed with rest of declaration. */
+			n->prodList->transfer( combined );
+		}
+
 		LangEl *langEl = declareLangEl( pd, this, n->name, LangEl::NonTerm );
-		//$$->langEl = langEl;
 
 		/* Get the language element. */
 		langEl->objectDef = n->objectDef;
 		langEl->reduceFirst = n->reduceFirst;
 		langEl->contextIn = n->contextIn;
-		langEl->defList.transfer( *n->defList );
+		langEl->prodList.transfer( *n->prodList );
 
-		for ( LelDefList::Iter d = langEl->defList; d.lte(); d++ ) {
+		for ( LelProdList::Iter d = langEl->prodList; d.lte(); d++ ) {
 			d->prodName = langEl;
 
 			if ( d->redBlock != 0 ) {
@@ -494,7 +527,7 @@ void Namespace::declare( Compiler *pd )
 	for ( TypeAliasList::Iter ta = typeAliasList; ta.lte(); ta++ )
 		declareTypeAlias( pd, this, ta->name, ta->typeRef );
 
-	/* Go into child aliases. */
+	/* Go into child namespaces. */
 	for ( NamespaceVect::Iter c = childNamespaces; c.lte(); c++ )
 		(*c)->declare( pd );
 }
@@ -630,7 +663,7 @@ void Compiler::declareByteCode()
 	for ( FunctionList::Iter f = functionList; f.lte(); f++ )
 		declareFunction( f );
 
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		if ( prod->redBlock != 0 )
 			declareReductionCode( prod );
 	}
@@ -660,6 +693,16 @@ void Compiler::makeDefaultIterators()
 		objMethod->iterDef = triter;
 	}
 
+	{
+		UniqueType *anyRefUT = findUniqueType( TYPE_REF, anyLangEl );
+		ObjectMethod *objMethod = initFunction( uniqueTypeAny, rootNamespace, globalObjectDef, 
+				ObjectMethod::Call, "with_ignore", IN_HALT, IN_HALT, anyRefUT, true );
+
+		IterDef *triter = findIterDef( IterDef::WithIgnore );
+		objMethod->iterDef = triter;
+	}
+
+
 	/* Child iterator. */
 	{
 		UniqueType *anyRefUT = findUniqueType( TYPE_REF, anyLangEl );
@@ -888,6 +931,9 @@ void Compiler::declareInputFields( )
 	initFunction( uniqueTypeVoid, inputObj, ObjectMethod::Call, "close",
 			IN_INPUT_CLOSE_WC, IN_INPUT_CLOSE_WC, false );
 
+	initFunction( uniqueTypeVoid, inputObj, ObjectMethod::Call, "auto_trim",
+			IN_IINPUT_AUTO_TRIM_WC, IN_IINPUT_AUTO_TRIM_WC, uniqueTypeBool, false );
+
 	declareInputField( inputObj, 0 );
 }
 
@@ -910,6 +956,9 @@ void Compiler::declareStreamFields( )
 	initFunction( uniqueTypeVoid, streamObj, ObjectMethod::Call, "close",
 			IN_INPUT_CLOSE_WC, IN_INPUT_CLOSE_WC, false );
 
+	initFunction( uniqueTypeVoid, streamObj, ObjectMethod::Call, "auto_trim",
+			IN_INPUT_AUTO_TRIM_WC, IN_INPUT_AUTO_TRIM_WC, uniqueTypeBool, false );
+
 	declareStreamField( streamObj, 0 );
 }
 
@@ -1485,7 +1534,7 @@ void Compiler::initParserFunctions( GenericType *gen )
 	initFunction( gen->elUt, gen->objDef, ObjectMethod::ParseFinish, "eof",
 			IN_PARSE_FRAG_W, IN_PARSE_FRAG_W, true );
 
-	initFunction( uniqueTypeStream, gen->objDef, ObjectMethod::Call, "gets",
+	initFunction( uniqueTypeInput, gen->objDef, ObjectMethod::Call, "gets",
 			IN_GET_PARSER_STREAM, IN_GET_PARSER_STREAM, true );
 }
 
