@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2012 Adrian Thurston <thurston@colm.net>
+ * Copyright 2006-2018 Adrian Thurston <thurston@colm.net>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -59,28 +59,11 @@ void Compiler::generateExports()
 		"#define _EXPORTS_H\n"
 		"\n"
 		"#include <colm/colm.h>\n"
+		"#include <colm/tree.h>\n"
+		"#include <colm/colmex.h>\n"
 		"#include <string>\n"
 		"\n";
 	
-	out << 
-		"inline void appendString( colm_print_args *args, const char *data, int length )\n"
-		"{\n"
-		"	std::string *str = (std::string*)args->arg;\n"
-		"	*str += std::string( data, length );\n"
-		"}\n"
-		"\n";
-
-	out << 
-		"inline std::string printTreeStr( colm_program *prg, colm_tree *tree, bool trim )\n"
-		"{\n"
-		"	std::string str;\n"
-		"	colm_print_args printArgs = { &str, 1, 0, trim, &appendString, \n"
-		"			&colm_print_null, &colm_print_term_tree, &colm_print_null };\n"
-		"	colm_print_tree_args( prg, colm_vm_root(prg), &printArgs, tree );\n"
-		"	return str;\n"
-		"}\n"
-		"\n";
-
 	/* Declare. */
 	for ( LelList::Iter lel = langEls; lel.lte(); lel++ ) {
 		if ( lel->isEOF )
@@ -99,23 +82,20 @@ void Compiler::generateExports()
 
 		openNameSpace( out, lel->nspace );
 		out << "struct " << lel->fullName << "\n";
+		out << "	: public ExportTree\n";
 		out << "{\n";
-		out << "	std::string text() { return printTreeStr( __prg, __tree, true ); }\n";
-		out << "	colm_location *loc() { return colm_find_location( __prg, __tree ); }\n";
-		out << "	std::string text_notrim() { return printTreeStr( __prg, __tree, false ); }\n";
-		out << "	std::string text_ws() { return printTreeStr( __prg, __tree, false ); }\n";
-		out << "	colm_data *data() { return __tree->tokdata; }\n";
-		out << "	operator colm_tree *() { return __tree; }\n";
-		out << "	colm_program *__prg;\n";
-		out << "	colm_tree *__tree;\n";
+		out << "	static const int ID = " << lel->id << ";\n";
 
 		if ( mainReturnUT != 0 && mainReturnUT->langEl == lel ) {
 			out << "	" << lel->fullName <<
-					"( colm_program *prg ) : __prg(prg), __tree(returnVal(prg)) {}\n";
+					"( colm_program *prg ) : ExportTree( prg, returnVal(prg) ) {\n";
+			out << "    }\n";
 		}
 
 		out << "	" << lel->fullName <<
-				"( colm_program *prg, colm_tree *tree ) : __prg(prg), __tree(tree) {}\n";
+				"( colm_program *prg, colm_tree *tree ) : ExportTree( prg, tree ) {\n";
+
+		out << "}\n";
 
 		if ( lel->objectDef != 0 ) {
 			FieldList &fieldList = lel->objectDef->fieldList;
@@ -131,35 +111,21 @@ void Compiler::generateExports()
 		}
 
 		bool prodNames = false;
-		for ( LelDefList::Iter prod = lel->defList; prod.lte(); prod++ ) {
-			if ( prod->name.length() > 0 )
+		for ( LelProdList::Iter prod = lel->prodList; prod.lte(); prod++ ) {
+			if ( prod->_name.length() > 0 )
 				prodNames = true;
 		}
 
 		if ( prodNames ) {
 			out << "	enum prod_name {\n";
-			for ( LelDefList::Iter prod = lel->defList; prod.lte(); prod++ ) {
-				if ( prod->name.length() > 0 )
-					out << "\t\t" << prod->name << " = " << prod->prodNum << ",\n";
+			for ( LelProdList::Iter prod = lel->prodList; prod.lte(); prod++ ) {
+				if ( prod->_name.length() > 0 )
+					out << "\t\t" << prod->_name << " = " << prod->prodNum << ",\n";
 			}
 			out << "	};\n";
 			out << "	enum prod_name prodName() " <<
 					"{ return (enum prod_name)__tree->prod_num; }\n";
 		}
-		
-
-		if ( lel->isRepeat ) {
-			out << "	" << "int end() { return colm_repeat_end( __tree ); }\n";
-			out << "	" << lel->refName << " next();\n";
-			out << "	" << lel->repeatOf->refName << " value();\n";
-		}
-
-		if ( lel->isList ) {
-			out << "	" << "int last() { return colm_list_last( __tree ); }\n";
-			out << "	" << lel->refName << " next();\n";
-			out << "	" << lel->repeatOf->refName << " value();\n";
-		}
-
 
 		out << "};";
 		closeNameSpace( out, lel->nspace );
@@ -199,10 +165,18 @@ void Compiler::generateExportsImpl()
 {
 	ostream &out = *outStream;
 
+	char *headerFn = strdup( exportHeaderFn );
+	char *suffix = strstr( headerFn, ".pack" );
+	if ( suffix != 0 && strcmp( suffix, ".pack" ) == 0 )
+		*suffix = 0;
+
 	if ( exportHeaderFn != 0 )  {
-		out << "#include \"" << exportHeaderFn << "\"\n";
+		out << "#include \"" << headerFn << "\"\n";
 	}
 
+	out << "#include <colm/tree.h>\n";
+	out << "#include <string.h>\n";
+
 	/* Function implementations. */
 	for ( LelList::Iter lel = langEls; lel.lte(); lel++ ) {
 		if ( lel->objectDef != 0 ) {
@@ -240,26 +214,6 @@ void Compiler::generateExportsImpl()
 				}
 			}
 		}
-
-		if ( lel->isRepeat ) {
-			out << lel->refName << " " << lel->declName << "::" << " next"
-				"() { return " << lel->refName << 
-				"( __prg, colm_get_repeat_next( __tree ) ); }\n";
-
-			out << lel->repeatOf->refName << " " << lel->declName << "::" << " value"
-				"() { return " << lel->repeatOf->refName << 
-				"( __prg, colm_get_repeat_val( __tree ) ); }\n";
-		}
-
-		if ( lel->isList ) {
-			out << lel->refName << " " << lel->declName << "::" << " next"
-				"() { return " << lel->refName << 
-				"( __prg, colm_get_repeat_next( __tree ) ); }\n";
-
-			out << lel->repeatOf->refName << " " << lel->declName << "::" << " value"
-				"() { return " << lel->repeatOf->refName << 
-				"( __prg, colm_get_repeat_val( __tree ) ); }\n";
-		}
 	}
 
 	out << "\n";
