@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2012 Adrian Thurston <thurston@colm.net>
+ * Copyright 2006-2018 Adrian Thurston <thurston@colm.net>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -32,7 +32,6 @@
 
 #include <avltree.h>
 
-#include "config.h"
 #include "defs.h"
 #include "keyops.h"
 
@@ -68,6 +67,7 @@ extern bool gblLibrary;
 extern long gblActiveRealm;
 extern char machineMain[];
 extern const char *exportHeaderFn;
+extern bool rangeCrossesZero;
 
 struct colm_location;
 
