@@ -1,5 +1,5 @@
 /*
- * Copyright 2001-2012 Adrian Thurston <thurston@colm.net>
+ * Copyright 2001-2018 Adrian Thurston <thurston@colm.net>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -35,6 +35,7 @@ enum MarkType
 
 typedef unsigned long long Size;
 
+/* This key struct does not implement unsigned. */
 struct Key
 {
 private:
@@ -57,12 +58,8 @@ public:
 	Key( const Key &key ) : key(key.key) {}
 	Key( long key ) : key(key) {}
 
-	/* Returns the value used to represent the key. This value must be
-	 * interpreted based on signedness. */
 	long getVal() const { return key; };
 
-	/* Returns the key casted to a long long. This form of the key does not
-	 * require and signedness interpretation. */
 	long long getLongLong() const;
 
 	bool isUpper() const { return ( 'A' <= key && key <= 'Z' ); }
@@ -92,6 +89,7 @@ struct HostType
 {
 	const char *data1;
 	const char *data2;
+	bool isSigned;
 	long long minVal;
 	long long maxVal;
 	unsigned int size;
@@ -101,8 +99,7 @@ struct HostLang
 {
 	HostType *hostTypes;
 	int numHostTypes;
-	HostType *defaultAlphType;
-	bool explicitUnsigned;
+	int defaultHostType;
 };
 
 extern HostLang *hostLang;
@@ -116,9 +113,9 @@ struct KeyOps
 	KeyOps() : alphType(0) {}
 
 	Key minKey, maxKey;
-	HostType *alphType;
+	const HostType *alphType;
 
-	void setAlphType( HostType *alphType )
+	void setAlphType( const HostType *alphType )
 	{
 		this->alphType = alphType;
 		minKey = (long) alphType->minVal;
