@@ -62,7 +62,7 @@ struct LoadInit
 	void walkTokenList( token_list &TokenList );
 	void walkLexRegion( item &LexRegion );
 	void walkProdElList( String defName, ProdElList *list, prod_el_list &prodElList );
-	void walkProdList( String defName, LelDefList *list, prod_list &prodList );
+	void walkProdList( String defName, LelProdList *list, prod_list &prodList );
 	void walkDefinition( item &define );
 
 	/* Constructing statements needed to parse and export the input. */
