@@ -26,6 +26,7 @@
 #include <strings.h>
 #include <stdio.h>
 #include <stdbool.h>
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <iostream>
@@ -40,7 +41,7 @@
 #elif defined(LOAD_INIT)
 #include "loadinit.h"
 #else
-#include "loadcolm.h"
+#include "loadfinal.h"
 #endif
 
 using std::istream;
@@ -85,9 +86,11 @@ bool generateGraphviz = false;
 bool verbose = false;
 bool logging = false;
 bool branchPointInfo = false;
+bool run = false;
 bool addUniqueEmptyProductions = false;
 bool gblLibrary = false;
 long gblActiveRealm = 0;
+bool outputSpecifiedWithDashP = false;
 
 ArgsVector includePaths;
 ArgsVector libraryPaths;
@@ -100,12 +103,21 @@ void version();
 /* Total error count. */
 int gblErrorCount = 0;
 
+/*
+ * Alphabet Type for the parsing machinery. The trees/strings of parsed data
+ * all use char type. Currently we can support signed char, unsigned char or
+ * char. If changing this, the colm_alph_t type needs to change as well.
+ * Currently, this is a compile time change only. A colm binary currently
+ * connot be made to work with multiple alphabet types.
+ */
+
 HostType hostTypesC[] =
 {
-	{ "char", 0, CHAR_MIN, CHAR_MAX, sizeof(char) },
+	{ "unsigned", "char", false, 0, UCHAR_MAX, sizeof(unsigned char) },
 };
 
-HostLang hostLangC = { hostTypesC, 8, hostTypesC+0, true };
+
+HostLang hostLangC = { hostTypesC, 1, 0 };
 HostLang *hostLang = &hostLangC;
 
 /* Print the opening to an error in the input, then return the error ostream. */
@@ -176,17 +188,20 @@ void usage()
 "general:\n"
 "   -h, -H, -?, --help   print this usage and exit\n"
 "   -v --version         print version information and exit\n"
-"   -b <file>            write binary to <file>\n"
-"   -o <file>            write object to <file>\n"
+"   -b <ident>           use <ident> as name of C object encapulaing the program\n"
+"   -o <file>            if -c given, write C parse object to <file>,\n"
+"                        otherwise write binary to <file>\n"
+"   -p <file>            write C parse object to <file>\n"
 "   -e <file>            write C++ export header to <file>\n"
 "   -x <file>            write C++ export code to <file>\n"
 "   -m <file>            write C++ commit code to <file>\n"
 "   -a <file>            additional code file to include in output program\n"
-"   -E N=V               set a string value availabe in the program\n"
+"   -E N=V               set a string value available in the program\n"
 "   -I <path>            additional include path for the compiler\n"
 "   -i                   activate branchpoint information\n"
 "   -L <path>            additional library path for the linker\n"
 "   -l                   activate logging\n"
+"   -r                   run output program and replace process\n"
 "   -c                   compile only (don't produce binary)\n"
 "   -V                   print dot format (graphiz)\n"
 "   -d                   print verbose debug information\n"
@@ -201,7 +216,7 @@ void usage()
 void version()
 {
 	cout << "Colm version " VERSION << " " PUBDATE << endl <<
-			"Copyright (c) 2007-2019 by Adrian D. Thurston" << endl;
+			"Copyright (c) 2007-2020 by Dr. Adrian D. Thurston" << endl;
 }
 
 /* Scans a string looking for the file extension. If there is a file
@@ -406,13 +421,24 @@ void openCommit( )
 	}
 }
 
-void compileOutputCommand( const char *command )
+int compileOutputCommand( const char *command )
 {
 	if ( verbose )
 		cout << "compiling with: '" << command << "'" << endl;
 	int res = system( command );
 	if ( res != 0 )
 		error() << "there was a problem compiling the output" << endl;
+
+	return res;
+}
+
+void runOutputProgram()
+{
+	if ( verbose )
+		cout << "running output: '" << binaryFn << "'" << endl;
+
+	execl( binaryFn, binaryFn, NULL );
+	/* We shall never return here! */
 }
 
 void compileOutput( const char *argv0, const bool inSource, char *srcLocation )
@@ -459,7 +485,7 @@ void compileOutput( const char *argv0, const bool inSource, char *srcLocation )
 				" -I%s/../aapl"
 				" -I%s/include"
 				" -L%s"
-				" -Wl,-rpath=%s",
+				" %s/libcolm.a",
 				binaryFn, intermedFn, srcLocation,
 				srcLocation, location, location );
 	}
@@ -484,9 +510,12 @@ void compileOutput( const char *argv0, const bool inSource, char *srcLocation )
 		strcat( command, " -L" );
 		strcat( command, *lp );
 	}
-	strcat( command, " -lcolm" );
 
-	compileOutputCommand( command );
+	if ( !inSource )
+		strcat( command, " -lcolm" );
+
+	if( !compileOutputCommand( command ) && run )
+		runOutputProgram();
 
 	delete[] command;
 }
@@ -530,7 +559,7 @@ bool inSourceTree( const char *argv0, char *&location )
 
 void processArgs( int argc, const char **argv )
 {
-	ParamCheck pc( "cD:e:x:I:L:vdlio:S:M:vHh?-:sVa:m:b:E:", argc, argv );
+	ParamCheck pc( "p:cD:e:x:I:L:vdliro:S:M:vHh?-:sVa:m:b:E:", argc, argv );
 
 	while ( pc.check() ) {
 		switch ( pc.state ) {
@@ -555,6 +584,12 @@ void processArgs( int argc, const char **argv )
 			case 'i':
 				branchPointInfo = true;
 				break;
+			case 'r':
+				run = true;
+				break;
+			case 'p':
+				outputSpecifiedWithDashP = true;
+				/* fallthrough */
 			case 'o':
 				/* Output. */
 				if ( *pc.parameterArg == 0 )
@@ -699,7 +734,7 @@ int main(int argc, const char **argv)
 
 	if ( verbose )
 		gblActiveRealm = 0xffffffff;
-
+	
 	/* Bail on above errors. */
 	if ( gblErrorCount > 0 )
 		exit(1);
@@ -724,6 +759,11 @@ int main(int argc, const char **argv)
 	}
 #endif
 
+	if ( !gblLibrary && outputSpecifiedWithDashP ) {
+		error() << "-p option must be used with -c" << endl;
+	}
+
+
 	/* Bail on above errors. */
 	if ( gblErrorCount > 0 )
 		exit(1);
