@@ -38,15 +38,15 @@ void BaseParser::listElDef( String name )
 	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
 			name, pd->nextObjectId++ );
 
-	LelDefList *defList = new LelDefList;
+	LelProdList *prodList = new LelProdList;
 
 	Production *prod = BaseParser::production( InputLoc(),
 			new ProdElList, String(), false, 0, 0 );
 
-	prodAppend( defList, prod );
+	prodAppend( prodList, prod );
 
 	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
-	BaseParser::cflDef( ntDef, objectDef, defList );
+	BaseParser::cflDef( ntDef, objectDef, prodList );
 
 	/*
 	 * List element with the same name as containing context.
@@ -71,14 +71,14 @@ void BaseParser::mapElDef( String name, TypeRef *keyType )
 	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
 			name, pd->nextObjectId++ );
 
-	LelDefList *defList = new LelDefList;
+	LelProdList *prodList = new LelProdList;
 
 	Production *prod = BaseParser::production( InputLoc(),
 			new ProdElList, String(), false, 0, 0 );
-	prodAppend( defList, prod );
+	prodAppend( prodList, prod );
 
 	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
-	BaseParser::cflDef( ntDef, objectDef, defList );
+	BaseParser::cflDef( ntDef, objectDef, prodList );
 
 	/*
 	 * Same name as containing context.
@@ -563,33 +563,35 @@ LangStmt *BaseParser::exportStmt( ObjectField *objField,
 }
 
 
-void BaseParser::cflDef( NtDef *ntDef, ObjectDef *objectDef, LelDefList *defList )
+void BaseParser::cflDef( NtDef *ntDef, ObjectDef *objectDef, LelProdList *prodList )
 {
 	Namespace *nspace = curNspace();
 
 	ntDef->objectDef = objectDef;
-	ntDef->defList = defList;
+	ntDef->prodList = prodList;
 
 	nspace->ntDefList.append( ntDef );
 
 	/* Declare the captures in the object. */
-	for ( LelDefList::Iter prod = *defList; prod.lte(); prod++ ) {
-		for ( ProdElList::Iter pel = *prod->prodElList; pel.lte(); pel++ ) {
-			/* If there is a capture, create the field. */
-			if ( pel->captureField != 0 ) {
-				/* Might already exist. */
-				ObjectField *newOf = objectDef->rootScope->checkRedecl(
-						pel->captureField->name );
-				if ( newOf != 0 ) {
-					/* FIXME: check the types are the same. */
-				}
-				else {
-					newOf = pel->captureField;
-					newOf->typeRef = pel->typeRef;
-					objectDef->rootScope->insertField( newOf->name, newOf );
-				}
+	for ( LelProdList::Iter prod = *prodList; prod.lte(); prod++ ) {
+		if ( prod->prodElList != 0 ) {
+			for ( ProdElList::Iter pel = *prod->prodElList; pel.lte(); pel++ ) {
+				/* If there is a capture, create the field. */
+				if ( pel->captureField != 0 ) {
+					/* Might already exist. */
+					ObjectField *newOf = objectDef->rootScope->checkRedecl(
+							pel->captureField->name );
+					if ( newOf != 0 ) {
+						/* FIXME: check the types are the same. */
+					}
+					else {
+						newOf = pel->captureField;
+						newOf->typeRef = pel->typeRef;
+						objectDef->rootScope->insertField( newOf->name, newOf );
+					}
 
-				newOf->rhsVal.append( RhsVal( pel ) );
+					newOf->rhsVal.append( RhsVal( pel ) );
+				}
 			}
 		}
 	}
@@ -829,6 +831,9 @@ ProdElList *BaseParser::appendProdEl( ProdElList *prodElList, ProdEl *prodEl )
 PatternItemList *BaseParser::patListConcat( PatternItemList *list1,
 		PatternItemList *list2 )
 {
+	if ( list1 == 0 )
+		list1 = new PatternItemList();
+		
 	list1->append( *list2 );
 	delete list2;
 	return list1;
@@ -837,6 +842,9 @@ PatternItemList *BaseParser::patListConcat( PatternItemList *list1,
 ConsItemList *BaseParser::consListConcat( ConsItemList *list1,
 		ConsItemList *list2 )
 {
+	if ( list1 == 0 )
+		list1 = new ConsItemList();
+
 	list1->append( *list2 );
 	delete list2;
 	return list1;
@@ -929,11 +937,11 @@ void BaseParser::objVarDef( ObjectDef *objectDef, ObjectField *objField )
 	objectDef->rootScope->insertField( objField->name, objField );
 }
 
-LelDefList *BaseParser::prodAppend( LelDefList *defList, Production *definition )
+LelProdList *BaseParser::prodAppend( LelProdList *prodList, Production *definition )
 {
-	definition->prodNum = defList->length();
-	defList->append( definition );
-	return defList;
+	definition->prodNum = prodList->length();
+	prodList->append( definition );
+	return prodList;
 }
 
 LangExpr *BaseParser::construct( const InputLoc &loc, ObjectField *objField,
