@@ -285,7 +285,7 @@ void prepareLitString( String &result, bool &caseInsensitive,
 
 std::ostream &operator<<(std::ostream &out, const Token &token );
 
-typedef AvlMap< String, TokenInstance*, CmpStr > LiteralDict;
+typedef AvlMap< String, TokenInstance*, ColmCmpStr > LiteralDict;
 typedef AvlMapEl< String, TokenInstance* > LiteralDictEl;
 
 /* Store the value and type of a priority augmentation. */
@@ -316,7 +316,7 @@ struct LexDefinition
 };
 
 typedef Vector<String> StringVect;
-typedef CmpTable<String, CmpStr> CmpStrVect;
+typedef CmpTable<String, ColmCmpStr> CmpStrVect;
 
 struct NamespaceQual
 {
@@ -492,22 +492,23 @@ struct TokenInstance
 	TokenInstance *dupOf;
 };
 
-struct LelDefList;
+struct LelProdList;
 
 struct NtDef
 {
 	static NtDef *cons( const String &name, Namespace *nspace,
-		LelDefList *defList, ObjectDef *objectDef,
+		LelProdList *prodList, ObjectDef *objectDef,
 		StructDef *contextIn, bool reduceFirst )
 	{ 
 		NtDef *nt = new NtDef;
 
 		nt->name = name;
 		nt->nspace = nspace;
-		nt->defList = defList;
+		nt->prodList = prodList;
 		nt->objectDef = objectDef;
 		nt->contextIn = contextIn;
 		nt->reduceFirst = reduceFirst;
+		nt->isRedef = false;
 
 		return nt;
 	}
@@ -519,20 +520,22 @@ struct NtDef
 
 		nt->name = name;
 		nt->nspace = nspace;
-		nt->defList = 0;
+		nt->prodList = 0;
 		nt->objectDef = 0;
 		nt->contextIn = contextIn;
 		nt->reduceFirst = reduceFirst;
+		nt->isRedef = false;
 
 		return nt;
 	}
 
 	String name;
 	Namespace *nspace;
-	LelDefList *defList;
+	LelProdList *prodList;
 	ObjectDef *objectDef;
 	StructDef *contextIn;
 	bool reduceFirst;
+	bool isRedef;
 
 	NtDef *prev, *next;
 };
@@ -623,7 +626,7 @@ struct TypeMapEl
 };
 
 /* Symbol Map. */
-typedef AvlTree< TypeMapEl, String, CmpStr > TypeMap;
+typedef AvlTree< TypeMapEl, String, ColmCmpStr > TypeMap;
 
 typedef Vector<TokenRegion*> RegionVect;
 
@@ -793,7 +796,7 @@ struct GraphDictEl
 	InputLoc loc;
 };
 
-typedef AvlTree<GraphDictEl, String, CmpStr> GraphDict;
+typedef AvlTree<GraphDictEl, String, ColmCmpStr> GraphDict;
 typedef DList<GraphDictEl> GraphList;
 
 struct TypeAlias
@@ -817,10 +820,10 @@ struct TypeAlias
 
 typedef DList<TypeAlias> TypeAliasList;
 
-typedef AvlMap<String, ObjectField*, CmpStr> FieldMap;
+typedef AvlMap<String, ObjectField*, ColmCmpStr> FieldMap;
 typedef AvlMapEl<String, ObjectField*> FieldMapEl;
 
-typedef AvlMap<String, ObjectMethod*, CmpStr> MethodMap;
+typedef AvlMap<String, ObjectMethod*, ColmCmpStr> MethodMap;
 typedef AvlMapEl<String, ObjectMethod*> MethodMapEl;
 
 /* tree_t of name scopes for an object def. All of the object fields inside this
@@ -1734,6 +1737,12 @@ struct PatternItemList
 
 struct ConsItem
 {
+	enum Trim {
+		TrimYes,
+		TrimNo,
+		TrimDefault
+	};
+
 	enum Type { 
 		InputText, 
 		ExprType,
@@ -1747,7 +1756,7 @@ struct ConsItem
 		langEl(0),
 		prodEl(0),
 		bindId(-1),
-		trim(false)
+		trim(TrimDefault)
 	{
 	}
 
@@ -1760,7 +1769,7 @@ struct ConsItem
 		return r;
 	}
 
-	static ConsItem *cons( const InputLoc &loc, Type type, LangExpr *expr, bool trim )
+	static ConsItem *cons( const InputLoc &loc, Type type, LangExpr *expr, Trim trim )
 	{
 		ConsItem *r = new ConsItem;
 		r->loc = loc;
@@ -1787,7 +1796,8 @@ struct ConsItem
 	LangEl *langEl;
 	ProdEl *prodEl;
 	long bindId;
-	bool trim;
+	Trim trim;
+
 	ConsItem *prev, *next;
 };
 
@@ -1924,7 +1934,7 @@ struct IterDef
 {
 	enum Type { Tree, Child, RevChild, Repeat,
 			RevRepeat, User, ListEl, 
-			RevListVal, MapEl };
+			RevListVal, MapEl, WithIgnore };
 
 	IterDef( Type type, Function *func );
 	IterDef( Type type );
@@ -1938,7 +1948,7 @@ struct IterImpl
 {
 	enum Type { Tree, Child, RevChild, Repeat,
 			RevRepeat, User, ListEl, ListVal,
-			RevListVal, MapEl, MapVal };
+			RevListVal, MapEl, MapVal, WithIgnore };
 
 	IterImpl( Type type, Function *func );
 	IterImpl( Type type );
@@ -2103,7 +2113,9 @@ enum RepeatType {
 	RepeatNone = 1,
 	RepeatRepeat,
 	RepeatList,
-	RepeatOpt
+	RepeatOpt,
+	RepeatLeftRepeat,
+	RepeatLeftList,
 };
 
 /*
