@@ -101,6 +101,7 @@ LangEl::LangEl( Namespace *nspace, const String &name, Type type )
 	isOpt(false),
 	parseStop(false),
 	isEOF(false),
+	leftRecursive(false),
 	repeatOf(0),
 	tokenDef(0),
 	rootDef(0),
@@ -171,7 +172,7 @@ void Compiler::makeDefinitionNames()
 {
 	for ( LelList::Iter lel = langEls; lel.lte(); lel++ ) {
 		int prodNum = 1;
-		for ( LelDefList::Iter def = lel->defList; def.lte(); def++ ) {
+		for ( LelProdList::Iter def = lel->prodList; def.lte(); def++ ) {
 			def->data.setAs( lel->name.length() + 32, "%s-%i", 
 					lel->name.data, prodNum++ );
 		}
@@ -302,14 +303,14 @@ void Compiler::makeProdFsms()
 	int dsiLow = 0, indexPos = 0;
 
 	/* Build FSMs for all production language elements. */
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ )
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ )
 		prod->fsm = prod->prodElList->walk( this, prod );
 
 	makeNonTermFirstSets();
 	makeFirstSets();
 
 	/* Build FSMs for all production language elements. */
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		if ( addUniqueEmptyProductions ) {
 			/* This must be re-implemented. */
 			assert( false );
@@ -365,7 +366,7 @@ void Compiler::makeProdFsms()
 
 	/* Make the final state specific prod id to prod id mapping. */
 	prodIdIndex = new Production*[prodList.length()];
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ )
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ )
 		prodIdIndex[prod->prodId] = prod;
 }
 
@@ -397,7 +398,7 @@ void Compiler::findFollow( AlphSet &result, PdaState *overTab,
 		LangEl *langEl = langElIndex[pastTrans->key];
 		if ( langEl != 0 && langEl->type == LangEl::NonTerm ) {
 			bool hasEpsilon = false;
-			for ( LelDefList::Iter def = langEl->defList; def.lte(); def++ ) {
+			for ( LelProdList::Iter def = langEl->prodList; def.lte(); def++ ) {
 				result.insert( def->firstSet );
 
 				if ( def->firstSet.find( -1 ) )
@@ -588,18 +589,18 @@ void Compiler::pdaOrderProd( LangEl *rootEl, PdaState *tabState,
 				/* Use a shortest match ordering for the contents of this
 				 * nonterminal. Does follows for all productions first, then
 				 * goes down the productions. */
-				for ( LelDefList::Iter expDef = langEl->defList; expDef.lte(); expDef++ ) {
+				for ( LelProdList::Iter expDef = langEl->prodList; expDef.lte(); expDef++ ) {
 					pdaOrderFollow( rootEl, tabState, tabTrans, srcTrans->value, 
 							parentDef, expDef, time );
 				}
-				for ( LelDefList::Iter expDef = langEl->defList; expDef.lte(); expDef++ )
+				for ( LelProdList::Iter expDef = langEl->prodList; expDef.lte(); expDef++ )
 					pdaOrderProd( rootEl, tabState, expDef->fsm->startState, expDef, time );
 				
 			}
 			else {
 				/* The default action ordering. For each prod, goes down the
 				 * prod then sets the follow before going to the next prod. */
-				for ( LelDefList::Iter expDef = langEl->defList; expDef.lte(); expDef++ ) {
+				for ( LelProdList::Iter expDef = langEl->prodList; expDef.lte(); expDef++ ) {
 					pdaOrderProd( rootEl, tabState, expDef->fsm->startState, expDef, time );
 
 					pdaOrderFollow( rootEl, tabState, tabTrans, srcTrans->value, 
@@ -1033,7 +1034,7 @@ void Compiler::analyzeMachine( PdaGraph *pdaGraph, LangElSet &parserEls )
 
 	/* Compute the max prod length. */
 	pdaGraph->maxProdLen = 0;
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		if ( (unsigned)prod->fsmLength > pdaGraph->maxProdLen )
 			pdaGraph->maxProdLen = prod->fsmLength;
 	}
@@ -1081,9 +1082,9 @@ void Compiler::wrapNonTerminals()
 		ProdElList *prodElList = makeProdElList( lel );
 		lel->rootDef = Production::cons( InputLoc(), rootLangEl, 
 				prodElList, String(), false, 0,
-				prodList.length(), rootLangEl->defList.length() );
+				prodList.length(), rootLangEl->prodList.length() );
 		prodList.append( lel->rootDef );
-		rootLangEl->defList.append( lel->rootDef );
+		rootLangEl->prodList.append( lel->rootDef );
 
 		/* First resolve. */
 		for ( ProdElList::Iter prodEl = *prodElList; prodEl.lte(); prodEl++ )
@@ -1102,7 +1103,7 @@ bool Compiler::makeNonTermFirstSetProd( Production *prod, PdaState *state )
 
 			bool hasEpsilon = false;
 			LangEl *lel = langElIndex[trans->key];
-			for ( LelDefList::Iter ldef = lel->defList; ldef.lte(); ldef++ ) {
+			for ( LelProdList::Iter ldef = lel->prodList; ldef.lte(); ldef++ ) {
 				for ( ProdIdSet::Iter pid = ldef->nonTermFirstSet; 
 						pid.lte(); pid++ )
 				{
@@ -1138,7 +1139,7 @@ void Compiler::makeNonTermFirstSets()
 	bool modified = true;
 	while ( modified ) {
 		modified = false;
-		for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+		for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 			if ( prod->fsm->startState->isFinState() ) {
 				long *inserted = prod->nonTermFirstSet.insert( -1 );
 				if ( inserted != 0 )
@@ -1151,7 +1152,7 @@ void Compiler::makeNonTermFirstSets()
 		}
 	}
 
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		if ( prod->nonTermFirstSet.find( prod->prodName->id ) )
 			prod->isLeftRec = true;
 	}
@@ -1159,7 +1160,7 @@ void Compiler::makeNonTermFirstSets()
 
 void Compiler::printNonTermFirstSets()
 {
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		cerr << prod->data << ": ";
 		for ( ProdIdSet::Iter pid = prod->nonTermFirstSet; pid.lte(); pid++ )
 		{
@@ -1200,7 +1201,7 @@ bool Compiler::makeFirstSetProd( Production *prod, PdaState *state )
 
 			bool hasEpsilon = false;
 			LangEl *lel = langElIndex[trans->key];
-			for ( LelDefList::Iter ldef = lel->defList; ldef.lte(); ldef++ ) {
+			for ( LelProdList::Iter ldef = lel->prodList; ldef.lte(); ldef++ ) {
 				for ( ProdIdSet::Iter pid = ldef->firstSet; 
 						pid.lte(); pid++ )
 				{
@@ -1236,7 +1237,7 @@ void Compiler::makeFirstSets()
 	bool modified = true;
 	while ( modified ) {
 		modified = false;
-		for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+		for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 			if ( prod->fsm->startState->isFinState() ) {
 				long *inserted = prod->firstSet.insert( -1 );
 				if ( inserted != 0 )
@@ -1252,7 +1253,7 @@ void Compiler::makeFirstSets()
 
 void Compiler::printFirstSets()
 {
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		cerr << prod->data << ": ";
 		for ( ProdIdSet::Iter pid = prod->firstSet; pid.lte(); pid++ )
 		{
@@ -1273,7 +1274,7 @@ void Compiler::printFirstSets()
 void Compiler::insertUniqueEmptyProductions()
 {
 	int limit = prodList.length();
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		if ( prod->prodId == limit )
 			break;
 
@@ -1282,8 +1283,8 @@ void Compiler::insertUniqueEmptyProductions()
 		sprintf(name, "U%li", prodList.length());
 		LangEl *prodName = addLangEl( this, rootNamespace, name, LangEl::NonTerm );
 		Production *newDef = Production::cons( InputLoc(), prodName, 
-				0, String(), false, 0, prodList.length(), prodName->defList.length() );
-		prodName->defList.append( newDef );
+				0, String(), false, 0, prodList.length(), prodName->prodList.length() );
+		prodName->prodList.append( newDef );
 		prodList.append( newDef );
 
 		prod->uniqueEmptyLeader = prodName;
@@ -1379,7 +1380,7 @@ void Compiler::makeRuntimeData()
 	runtimeData->num_prods = count;
 
 	count = 0;
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		runtimeData->prod_info[count].lhs_id = prod->prodName->id;
 		runtimeData->prod_info[count].prod_num = prod->prodNum;
 		runtimeData->prod_info[count].length = prod->fsmLength;
@@ -1454,6 +1455,7 @@ void Compiler::makeRuntimeData()
 	for ( int i = 0; i < nextLelId; i++ ) {
 		LangEl *lel = langElIndex[i];
 		if ( lel != 0 ) {
+			assert( lel->id == i );
 			runtimeData->lel_info[i].name = lel->fullLit;
 			runtimeData->lel_info[i].xml_tag = lel->xmlTag;
 			runtimeData->lel_info[i].repeat = lel->isRepeat;
@@ -1680,6 +1682,36 @@ void Compiler::makeRuntimeData()
 
 	runtimeData->global_size = globalObjectDef->size();
 
+	/*
+	 * Exports.
+	 */
+	count = 0;
+	for ( FieldList::Iter of = globalObjectDef->fieldList; of.lte(); of++ ) {
+		ObjectField *field = of->value;
+		if ( field->isExport ) {
+			count += 1;
+		}
+	}
+
+	runtimeData->num_exports = count;
+	if ( count == 0 ) {
+		runtimeData->export_info = 0;
+	}
+	else {
+		runtimeData->export_info = new export_info[count];
+
+		long i = 0;
+		for ( FieldList::Iter of = globalObjectDef->fieldList; of.lte(); of++ ) {
+			ObjectField *field = of->value;
+			if ( field->isExport ) {
+				runtimeData->export_info[i].name = strdup( field->name );
+				runtimeData->export_info[i].global_id = field->offset;
+				i += 1;
+			}
+		}
+	}
+
+
 	/*
 	 * Boundary between terms and non-terms.
 	 */
@@ -1973,7 +2005,7 @@ struct CmpSpan
 
 PdaGraph *Compiler::makePdaGraph( LangElSet &parserEls )
 {
-	//for ( DefList::Iter prod = prodList; prod.lte(); prod++ )
+	//for ( ProdList::Iter prod = prodList; prod.lte(); prod++ )
 	//	cerr << prod->prodId << " " << prod->data << endl;
 
 	PdaGraph *pdaGraph = new PdaGraph();
@@ -2006,12 +2038,12 @@ struct pda_tables *Compiler::makePdaTables( PdaGraph *pdaGraph )
 	}
 
 
-	/* Allocate indicies and owners. */
-	pdaTables->num_indicies = count;
-	pdaTables->indicies = new int[count];
+	/* Allocate indices and owners. */
+	pdaTables->num_indices = count;
+	pdaTables->indices = new int[count];
 	pdaTables->owners = new int[count];
 	for ( long i = 0; i < count; i++ ) {
-		pdaTables->indicies[i] = -1;
+		pdaTables->indices[i] = -1;
 		pdaTables->owners[i] = -1;
 	}
 
@@ -2020,7 +2052,7 @@ struct pda_tables *Compiler::makePdaTables( PdaGraph *pdaGraph )
 	pdaTables->offsets = new unsigned int[numStates];
 	pdaTables->num_states = numStates;
 
-	/* Place transitions into indicies/owners */
+	/* Place transitions into indices/owners */
 	PdaState **states = new PdaState*[numStates];
 	long ds = 0;
 	for ( PdaStateList::Iter state = pdaGraph->stateList; state.lte(); state++ )
@@ -2039,7 +2071,7 @@ struct pda_tables *Compiler::makePdaTables( PdaGraph *pdaGraph )
 		pdaTables->offsets[state->stateNum] = indOff;
 
 		for ( TransMap::Iter trans = state->transMap; trans.lte(); trans++ ) {
-			pdaTables->indicies[indOff] = trans->value->actionSetEl->key.id;
+			pdaTables->indices[indOff] = trans->value->actionSetEl->key.id;
 			pdaTables->owners[indOff] = state->stateNum;
 			indOff++;
 
@@ -2054,7 +2086,7 @@ struct pda_tables *Compiler::makePdaTables( PdaGraph *pdaGraph )
 	}
 
 	/* We allocated the max, but cmpression gives us less. */
-	pdaTables->num_indicies = indLen;
+	pdaTables->num_indices = indLen;
 	delete[] states;
 	
 
