@@ -23,6 +23,7 @@
 #include <string.h>
 
 #include <iostream>
+#include <iomanip>
 
 #include "compiler.h"
 #include "pdacodegen.h"
@@ -165,7 +166,7 @@ void PdaCodeGen::writeRuntimeData( colm_sections *runtimeData, struct pda_tables
 	out << "static struct lang_el_info " << lelInfo() << "[] = {\n";
 	for ( int i = 0; i < runtimeData->num_lang_els; i++ ) {
 		struct lang_el_info *el = &runtimeData->lel_info[i];
-		out << "\t{";
+		out << "/* " << std::setw(4) << i << " */ {";
 		
 		/* Name. */
 		out << " \"";
@@ -397,7 +398,7 @@ void PdaCodeGen::writeRuntimeData( colm_sections *runtimeData, struct pda_tables
 	out << "static const char *" << litdata() << "[] = {\n";
 	for ( int i = 0; i < runtimeData->num_literals; i++ ) {
 		out << "\t\"";
-		escapeLiteralString( out, runtimeData->litdata[i] );
+		escapeLiteralString( out, runtimeData->litdata[i], runtimeData->litlen[i] );
 		out << "\",\n";
 	}
 	out << "};\n\n";
@@ -453,6 +454,24 @@ void PdaCodeGen::writeRuntimeData( colm_sections *runtimeData, struct pda_tables
 		"void " << objectName << "_read_reduce( program_t *prg, int reducer, input_t *stream );\n"
 		"\n";
 
+	out <<
+		"static struct export_info " << exportInfo() << "[] = {\n";
+
+	for ( long i = 0; i < runtimeData->num_exports; i++ ) {
+		out << "	{ \"" << runtimeData->export_info[i].name << "\", " <<
+				runtimeData->export_info[i].global_id << " },\n";
+	}
+
+	out <<
+		"};\n";
+
+	for ( long i = 0; i < runtimeData->num_exports; i++ ) {
+		out << "const int colm_export_" << runtimeData->export_info[i].name << " = " <<
+				runtimeData->export_info[i].global_id << ";\n";
+	}
+	out <<
+		"\n";
+
 	out <<
 		"struct colm_sections " << objectName << " = \n"
 		"{\n"
@@ -486,6 +505,10 @@ void PdaCodeGen::writeRuntimeData( colm_sections *runtimeData, struct pda_tables
 		"\n"
 		"	" << genericInfo() << ",\n"
 		"	" << runtimeData->num_generics << ",\n"
+		"\n"
+		"	" << exportInfo() << ",\n"
+		"	" << runtimeData->num_exports << ",\n"
+		"\n"
 		"	" << runtimeData->argv_generic_id << ",\n"
 		"	" << runtimeData->stds_generic_id << ",\n"
 		"\n"
@@ -535,11 +558,11 @@ void PdaCodeGen::writeParserData( long id, struct pda_tables *tables )
 {
 	String prefix = "pid_" + String(0, "%ld", id) + "_";
 
-	out << "static int " << prefix << indicies() << "[] = {\n\t";
-	for ( int i = 0; i < tables->num_indicies; i++ ) {
-		out << tables->indicies[i];
+	out << "static int " << prefix << indices() << "[] = {\n\t";
+	for ( int i = 0; i < tables->num_indices; i++ ) {
+		out << tables->indices[i];
 
-		if ( i < tables->num_indicies-1 ) {
+		if ( i < tables->num_indices-1 ) {
 			out << ", ";
 			if ( (i+1) % 8 == 0 )
 				out << "\n\t";
@@ -548,10 +571,10 @@ void PdaCodeGen::writeParserData( long id, struct pda_tables *tables )
 	out << "\n};\n\n";
 
 	out << "static int " << prefix << owners() << "[] = {\n\t";
-	for ( int i = 0; i < tables->num_indicies; i++ ) {
+	for ( int i = 0; i < tables->num_indices; i++ ) {
 		out << tables->owners[i];
 
-		if ( i < tables->num_indicies-1 ) {
+		if ( i < tables->num_indices-1 ) {
 			out << ", ";
 			if ( (i+1) % 8 == 0 )
 				out << "\n\t";
@@ -670,7 +693,7 @@ void PdaCodeGen::writeParserData( long id, struct pda_tables *tables )
 	out << 
 		"static struct pda_tables " << prefix << "pdaTables =\n"
 		"{\n"
-		"	" << prefix << indicies() << ",\n"
+		"	" << prefix << indices() << ",\n"
 		"	" << prefix << owners() << ",\n"
 		"	" << prefix << keys() << ",\n"
 		"	" << prefix << offsets() << ",\n"
@@ -683,7 +706,7 @@ void PdaCodeGen::writeParserData( long id, struct pda_tables *tables )
 		"	" << prefix << tokenRegions() << ",\n"
 		"	" << prefix << tokenPreRegions() << ",\n"
 		"\n"
-		"	" << tables->num_indicies << ",\n"
+		"	" << tables->num_indices << ",\n"
 		"	" << tables->num_keys << ",\n"
 		"	" << tables->num_states << ",\n"
 		"	" << tables->num_targs << ",\n"
