@@ -69,7 +69,7 @@ static void init_fsm_run( program_t *prg, struct pda_run *pda_run )
 	pda_run->consume_buf = 0;
 
 	pda_run->p = pda_run->pe = 0;
-	pda_run->toklen = 0;
+	pda_run->tokpref = 0;
 	pda_run->scan_eof = 0;
 
 	pda_run->pre_region = -1;
@@ -112,7 +112,7 @@ head_t *colm_stream_pull( program_t *prg, tree_t **sp, struct pda_run *pda_run,
 			pda_run->consume_buf = run_buf;
 		}
 
-		char *dest = run_buf->data + run_buf->length;
+		alph_t *dest = run_buf->data + run_buf->length;
 
 		is->funcs->get_data( prg, is, dest, length );
 		location_t *loc = location_allocate( prg );
@@ -121,16 +121,16 @@ head_t *colm_stream_pull( program_t *prg, tree_t **sp, struct pda_run *pda_run,
 		run_buf->length += length;
 
 		pda_run->p = pda_run->pe = 0;
-		pda_run->toklen = 0;
+		pda_run->tokpref = 0;
 
-		head_t *tokdata = colm_string_alloc_pointer( prg, dest, length );
+		head_t *tokdata = colm_string_alloc_pointer( prg, colm_cstr_from_alph( dest ), length );
 		tokdata->location = loc;
 
 		return tokdata;
 	}
 	else {
 		head_t *head = init_str_space( length );
-		char *dest = (char*)head->data;
+		alph_t *dest = (alph_t*)head->data;
 
 		is->funcs->get_data( prg, is, dest, length );
 		location_t *loc = location_allocate( prg );
@@ -141,36 +141,10 @@ head_t *colm_stream_pull( program_t *prg, tree_t **sp, struct pda_run *pda_run,
 	}
 }
 
-void colm_stream_push_text( struct colm_program *prg, struct input_impl *is, const char *data, long length )
-{
-	is->funcs->prepend_data( prg, is, data, length );
-}
-
-void colm_stream_push_tree( struct colm_program *prg, struct input_impl *is, tree_t *tree, int ignore )
-{
-	is->funcs->prepend_tree( prg, is, tree, ignore );
-}
-
-void colm_stream_push_stream( struct colm_program *prg, struct input_impl *is, stream_t *stream )
-{
-	is->funcs->prepend_stream( prg, is, stream );
-}
-
-void colm_undo_stream_push( program_t *prg, tree_t **sp, struct input_impl *is, long length )
-{
-	if ( length < 0 ) {
-		/* tree_t *tree = */ is->funcs->undo_prepend_tree( prg, is );
-		// colm_tree_downref( prg, sp, tree );
-	}
-	else {
-		is->funcs->undo_prepend_data( prg, is, length );
-	}
-}
-
 /* Should only be sending back whole tokens/ignores, therefore the send back
  * should never cross a buffer boundary. Either we slide back data, or we move to
  * a previous buffer and slide back data. */
-static void send_back_text( struct colm_program *prg, struct input_impl *is, const char *data, long length )
+static void send_back_text( struct colm_program *prg, struct input_impl *is, const alph_t *data, long length )
 {
 	//debug( REALM_PARSE, "push back of %ld characters\n", length );
 
@@ -206,10 +180,12 @@ static void send_back_ignore( program_t *prg, tree_t **sp,
 	int artificial = parse_tree->flags & PF_ARTIFICIAL;
 
 	if ( head != 0 ) {
-		if ( artificial )
+		if ( artificial ) {
+			colm_tree_upref( prg, parse_tree->shadow->tree );
 			send_back_tree( prg, is, parse_tree->shadow->tree );
+		}
 		else
-			send_back_text( prg, is, string_data( head ), head->length );
+			send_back_text( prg, is, colm_alph_from_cstr( string_data( head ) ), head->length );
 	}
 
 	colm_decrement_steps( pda_run );
@@ -233,7 +209,7 @@ static void reset_token( struct pda_run *pda_run )
 	 * must first backup over it. */
 	if ( pda_run->tokstart != 0 ) {
 		pda_run->p = pda_run->pe = 0;
-		pda_run->toklen = 0;
+		pda_run->tokpref = 0;
 		pda_run->scan_eof = 0;
 	}
 }
@@ -266,7 +242,6 @@ static void send_back( program_t *prg, tree_t **sp, struct pda_run *pda_run,
 		}
 
 		colm_tree_upref( prg, parse_tree->shadow->tree );
-
 		send_back_tree( prg, is, parse_tree->shadow->tree );
 	}
 	else {
@@ -278,7 +253,7 @@ static void send_back( program_t *prg, tree_t **sp, struct pda_run *pda_run,
 		}
 
 		/* Push back the token data. */
-		send_back_text( prg, is, string_data( parse_tree->shadow->tree->tokdata ), 
+		send_back_text( prg, is, colm_alph_from_cstr( string_data( parse_tree->shadow->tree->tokdata ) ), 
 				string_length( parse_tree->shadow->tree->tokdata ) );
 
 		/* If eof was just sent back remember that it needs to be sent again. */
@@ -377,7 +352,7 @@ kid_t *make_token_with_data( program_t *prg, struct pda_run *pda_run,
 		for ( i = 0; i < lel_info[id].num_capture_attr; i++ ) {
 			CaptureAttr *ca = &prg->rtd->capture_attr[lel_info[id].capture_attr + i];
 			head_t *data = string_alloc_full( prg, 
-					pda_run->mark[ca->mark_enter],
+					colm_cstr_from_alph( pda_run->mark[ca->mark_enter]  ),
 					pda_run->mark[ca->mark_leave] -
 							pda_run->mark[ca->mark_enter] );
 			tree_t *string = construct_string( prg, data );
@@ -725,7 +700,7 @@ static void handle_error( program_t *prg, tree_t **sp, struct pda_run *pda_run )
 static head_t *extract_match( program_t *prg, tree_t **sp,
 		struct pda_run *pda_run, struct input_impl *is )
 {
-	long length = pda_run->toklen;
+	long length = pda_run->tokend;
 
 	//debug( prg, REALM_PARSE, "extracting token of length: %ld\n", length );
 
@@ -736,19 +711,19 @@ static head_t *extract_match( program_t *prg, tree_t **sp,
 		pda_run->consume_buf = run_buf;
 	}
 
-	char *dest = run_buf->data + run_buf->length;
+	alph_t *dest = run_buf->data + run_buf->length;
 
-	is->funcs->get_data( prg, is, dest, length );
+	is->funcs->get_data( prg, is, (alph_t*)dest, length );
 	location_t *location = location_allocate( prg );
 	is->funcs->consume_data( prg, is, length, location );
 
 	run_buf->length += length;
 
 	pda_run->p = pda_run->pe = 0;
-	pda_run->toklen = 0;
+	pda_run->tokpref = 0;
 	pda_run->tokstart = 0;
 
-	head_t *head = colm_string_alloc_pointer( prg, dest, length );
+	head_t *head = colm_string_alloc_pointer( prg, colm_cstr_from_alph( dest ), length );
 
 	head->location = location;
 
@@ -760,14 +735,14 @@ static head_t *extract_match( program_t *prg, tree_t **sp,
 static head_t *extract_no_d( program_t *prg, tree_t **sp,
 		struct pda_run *pda_run, struct input_impl *is )
 {
-	long length = pda_run->toklen;
+	long length = pda_run->tokend;
 
 	/* Just a consume, no data allocate. */
 	location_t *location = location_allocate( prg );
 	is->funcs->consume_data( prg, is, length, location );
 
 	pda_run->p = pda_run->pe = 0;
-	pda_run->toklen = 0;
+	pda_run->tokpref = 0;
 	pda_run->tokstart = 0;
 
 	head_t *head = colm_string_alloc_pointer( prg, 0, 0 );
@@ -782,7 +757,7 @@ static head_t *extract_no_d( program_t *prg, tree_t **sp,
 static head_t *extract_no_l( program_t *prg, tree_t **sp,
 		struct pda_run *pda_run, struct input_impl *is )
 {
-	long length = pda_run->toklen;
+	long length = pda_run->tokend;
 
 	//debug( prg, REALM_PARSE, "extracting token of length: %ld\n", length );
 
@@ -793,7 +768,7 @@ static head_t *extract_no_l( program_t *prg, tree_t **sp,
 		pda_run->consume_buf = run_buf;
 	}
 
-	char *dest = run_buf->data + run_buf->length;
+	alph_t *dest = run_buf->data + run_buf->length;
 
 	is->funcs->get_data( prg, is, dest, length );
 
@@ -805,10 +780,10 @@ static head_t *extract_no_l( program_t *prg, tree_t **sp,
 	run_buf->length += length;
 
 	pda_run->p = pda_run->pe = 0;
-	pda_run->toklen = 0;
+	pda_run->tokpref = 0;
 	pda_run->tokstart = 0;
 
-	head_t *head = colm_string_alloc_pointer( prg, dest, length );
+	head_t *head = colm_string_alloc_pointer( prg, colm_cstr_from_alph( dest ), length );
 
 	/* Don't pass the location. */
 	head->location = 0;
@@ -821,7 +796,7 @@ static head_t *extract_no_l( program_t *prg, tree_t **sp,
 static head_t *consume_match( program_t *prg, tree_t **sp,
 		struct pda_run *pda_run, struct input_impl *is )
 {
-	long length = pda_run->toklen;
+	long length = pda_run->tokend;
 
 	/* No data or location returned. We just consume the data. */
 	location_t dummy_loc;
@@ -829,7 +804,7 @@ static head_t *consume_match( program_t *prg, tree_t **sp,
 	is->funcs->consume_data( prg, is, length, &dummy_loc );
 
 	pda_run->p = pda_run->pe = 0;
-	pda_run->toklen = 0;
+	pda_run->tokpref = 0;
 	pda_run->tokstart = 0;
 
 	debug( prg, REALM_PARSE, "location byte: %d\n", dummy_loc.byte );
@@ -840,7 +815,7 @@ static head_t *consume_match( program_t *prg, tree_t **sp,
 
 static head_t *peek_match( program_t *prg, struct pda_run *pda_run, struct input_impl *is )
 {
-	long length = pda_run->toklen;
+	long length = pda_run->tokend;
 
 	struct run_buf *run_buf = pda_run->consume_buf;
 	if ( run_buf == 0 || length > ( FSM_BUFSIZE - run_buf->length ) ) {
@@ -849,14 +824,14 @@ static head_t *peek_match( program_t *prg, struct pda_run *pda_run, struct input
 		pda_run->consume_buf = run_buf;
 	}
 
-	char *dest = run_buf->data + run_buf->length;
+	alph_t *dest = run_buf->data + run_buf->length;
 
 	is->funcs->get_data( prg, is, dest, length );
 
 	pda_run->p = pda_run->pe = 0;
-	pda_run->toklen = 0;
+	pda_run->tokpref = 0;
 
-	head_t *head = colm_string_alloc_pointer( prg, dest, length );
+	head_t *head = colm_string_alloc_pointer( prg, colm_cstr_from_alph( dest ), length );
 
 	head->location = location_allocate( prg );
 	is->funcs->transfer_loc( prg, head->location, is );
@@ -1032,7 +1007,7 @@ static void send_eof( program_t *prg, tree_t **sp, struct pda_run *pda_run,
 static void new_token( program_t *prg, struct pda_run *pda_run )
 {
 	pda_run->p = pda_run->pe = 0;
-	pda_run->toklen = 0;
+	pda_run->tokpref = 0;
 	pda_run->scan_eof = 0;
 
 	/* Init the scanner vars. */
@@ -1093,10 +1068,10 @@ static long scan_token( program_t *prg, struct pda_run *pda_run, struct input_im
 		return SCAN_UNDO;
 
 	while ( true ) {
-		char *pd = 0;
+		alph_t *pd = 0;
 		int len = 0;
-		int toklen = pda_run->toklen;
-		int type = is->funcs->get_parse_block( prg, is, &toklen, &pd, &len );
+		int tokpref = pda_run->tokpref;
+		int type = is->funcs->get_parse_block( prg, is, &tokpref, &pd, &len );
 
 		switch ( type ) {
 			case INPUT_DATA:
@@ -1167,7 +1142,7 @@ static long scan_token( program_t *prg, struct pda_run *pda_run, struct input_im
 			/* Check for a default token in the region. If one is there
 			 * then send it and continue with the processing loop. */
 			if ( prg->rtd->region_info[pda_run->region].default_token >= 0 ) {
-				pda_run->toklen = 0;
+				pda_run->tokpref = 0;
 				return prg->rtd->region_info[pda_run->region].default_token;
 			}
 
@@ -1372,7 +1347,7 @@ static long stack_top_target( program_t *prg, struct pda_run *pda_run )
 		unsigned shift = pda_run->stack_top->id - 
 				pda_run->pda_tables->keys[pda_run->stack_top->state<<1];
 		unsigned offset = pda_run->pda_tables->offsets[pda_run->stack_top->state] + shift;
-		int index = pda_run->pda_tables->indicies[offset];
+		int index = pda_run->pda_tables->indices[offset];
 		state = pda_run->pda_tables->targs[index];
 	}
 	return state;
@@ -1439,7 +1414,7 @@ again:
 		goto parse_error;
 	}
 
-	pos = pda_run->pda_tables->indicies[ind_pos];
+	pos = pda_run->pda_tables->indices[ind_pos];
 	if ( pos < 0 ) {
 		debug( prg, REALM_PARSE, "parse error, no transition 3\n" );
 		push_bt_point( prg, pda_run );
@@ -1981,6 +1956,7 @@ long colm_parse_loop( program_t *prg, tree_t **sp, struct pda_run *pda_run,
 			if ( pda_run->pre_region >= 0 ) {
 				pda_run->pre_region = -1;
 				pda_run->fsm_cs = pda_run->next_cs;
+				pda_run->tokpref = 0;
 				continue;
 			}
 		}
@@ -2086,7 +2062,7 @@ long colm_parse_loop( program_t *prg, tree_t **sp, struct pda_run *pda_run,
 			 * data is pulled from the inputStream. */
 
 			pda_run->p = pda_run->pe = 0;
-			pda_run->toklen = 0;
+			pda_run->tokpref = 0;
 			pda_run->scan_eof = 0;
 
 			pda_run->fi = &prg->rtd->frame_info[prg->rtd->lel_info[pda_run->token_id].frame_id];
