@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2012 Adrian Thurston <thurston@colm.net>
+ * Copyright 2006-2018 Adrian Thurston <thurston@colm.net>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -382,8 +382,8 @@ void RedFsmBuild::newAction( int anum, char *name, int line, int col, Action *ac
 {
 	redFsm->allActions[anum].actionId = anum;
 	redFsm->allActions[anum].name = name;
-	redFsm->allActions[anum].loc.line = line;
-	redFsm->allActions[anum].loc.col = col;
+	redFsm->allActions[anum].loc_line = line;
+	redFsm->allActions[anum].loc_col = col;
 	redFsm->allActions[anum].inlineList = action->inlineList;
 	redFsm->allActions[anum].objField = action->objField;
 	redFsm->allActions[anum].markType = action->markType;
