@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2012 Adrian Thurston <thurston@colm.net>
+ * Copyright 2006-2018 Adrian Thurston <thurston@colm.net>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -60,7 +60,8 @@ struct ObjectField;
 struct GenAction 
 {
 	/* Data collected during parse. */
-	InputLoc loc;
+	int loc_line;
+	int loc_col;
 	char *name;
 	InlineList *inlineList;
 	int actionId;
