@@ -114,7 +114,7 @@ UniqueType *TypeRef::resolveTypeLiteral( Compiler *pd )
 		nspace = nspace->parentNamespace;
 	}
 
-	error(loc) << "unknown type " << typeName << " in typeof expression" << endp;
+	error(loc) << "unknown type " << pdaLiteral->data << " in typeof expression" << endp;
 	return 0;
 }
 
@@ -340,7 +340,6 @@ UniqueType *TypeRef::resolveTypeParser( Compiler *pd )
  * End object based list/map
  */
 
-
 UniqueType *TypeRef::resolveTypeRef( Compiler *pd )
 {
 	typeRef1->resolveType( pd );
@@ -365,14 +364,28 @@ void TypeRef::resolveRepeat( Compiler *pd )
 				/* If the factor is a repeat, create the repeat element and link the
 				 * factor to it. */
 				String repeatName( 128, "_repeat_%s", typeName.data );
-				declLangEl = pd->makeRepeatProd( loc, nspace, repeatName, uniqueType );
+				declLangEl = pd->makeRepeatProd( loc, nspace, repeatName, uniqueType, false );
+				break;
+			}
+			case RepeatLeftRepeat: {
+				/* If the factor is a repeat, create the repeat element and link the
+				 * factor to it. */
+				String repeatName( 128, "_lrepeat_%s", typeName.data );
+				declLangEl = pd->makeRepeatProd( loc, nspace, repeatName, uniqueType, true );
 				break;
 			}
 			case RepeatList: {
 				/* If the factor is a repeat, create the repeat element and link the
 				 * factor to it. */
 				String listName( 128, "_list_%s", typeName.data );
-				declLangEl = pd->makeListProd( loc, nspace, listName, uniqueType );
+				declLangEl = pd->makeListProd( loc, nspace, listName, uniqueType, false );
+				break;
+			}
+			case RepeatLeftList: {
+				/* If the factor is a repeat, create the repeat element and link the
+				 * factor to it. */
+				String repeatName( 128, "_llist_%s", typeName.data );
+				declLangEl = pd->makeListProd( loc, nspace, repeatName, uniqueType, true );
 				break;
 			}
 			case RepeatOpt: {
@@ -382,7 +395,6 @@ void TypeRef::resolveRepeat( Compiler *pd )
 				declLangEl = pd->makeOptProd( loc, nspace, optName, uniqueType );
 				break;
 			}
-
 			case RepeatNone:
 				break;
 		}
@@ -762,7 +774,7 @@ void Compiler::resolveParseTree()
 		resolveInHost( f );
 
 	/* Compile the reduction code. */
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		if ( prod->redBlock != 0 )
 			resolveReductionCode( prod );
 	}
@@ -810,7 +822,7 @@ void Compiler::resolveParseTree()
 void Compiler::resolveProductionEls()
 {
 	/* NOTE: as we process this list it may be growing! */
-	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
+	for ( ProdList::Iter prod = prodList; prod.lte(); prod++ ) {
 		/* First resolve. */
 		for ( ProdElList::Iter prodEl = *prod->prodElList; prodEl.lte(); prodEl++ )
 			resolveProdEl( prodEl );
@@ -898,7 +910,7 @@ void Compiler::resolveReductionActions()
 
 Production *Compiler::findProductionByLabel( LangEl *langEl, String label )
 {
-	for ( LelDefList::Iter ldi = langEl->defList; ldi.lte(); ldi++ ) {
+	for ( LelProdList::Iter ldi = langEl->prodList; ldi.lte(); ldi++ ) {
 		if ( ldi->_name != 0 && ( strcmp( ldi->_name, label ) == 0 ) )
 			return ldi;
 	}
