@@ -20,6 +20,13 @@
  * SOFTWARE.
  */
 
+#include "config.h"
+
+#ifdef HAVE_FOPENCOOKIE
+#define _GNU_SOURCE
+#include <sys/types.h>
+#endif
+
 #include <colm/input.h>
 
 #include <stdio.h>
@@ -42,6 +49,57 @@ DEF_STREAM_FUNCS( stream_funcs_data, stream_impl_data );
 extern struct stream_funcs_data file_funcs;
 extern struct stream_funcs_data accum_funcs;
 
+#ifdef HAVE_FOPENCOOKIE
+
+struct colm_file_cookie
+{
+	int fd;
+};
+
+static ssize_t cfc_read(void *cookie, char *buf, size_t size)
+{
+	return read( ((struct colm_file_cookie*)cookie)->fd, buf, size );
+}
+
+static ssize_t cfc_write(void *cookie, const char *buf, size_t size)
+{
+	return write( ((struct colm_file_cookie*)cookie)->fd, buf, size );
+}
+
+static int cfc_seek(void *cookie, off_t *offset, int whence)
+{
+	return -1;
+}
+
+static int cfc_close(void *cookie)
+{
+	free(cookie);
+	return 0;
+}
+
+FILE *colm_fd_open( int fd, const char *mode )
+{
+	cookie_io_functions_t cf = {
+		cfc_read,
+		cfc_write,
+		cfc_seek,
+		cfc_close,
+	};
+
+	struct colm_file_cookie *cfc = malloc(sizeof(struct colm_file_cookie));
+	cfc->fd = fd;
+	return fopencookie( cfc, mode, cf );
+}
+
+#else
+
+FILE *colm_fd_open( int fd, const char *mode )
+{
+	return fdopen( fd, mode );
+}
+
+#endif
+
 void stream_impl_push_line( struct stream_impl_data *ss, int ll )
 {
 	if ( ss->line_len == 0 ) {
@@ -53,6 +111,7 @@ void stream_impl_push_line( struct stream_impl_data *ss, int ll )
 		int lines_alloc_new = ss->lines_alloc * 2;
 		int *line_len_new = malloc( sizeof(int) * lines_alloc_new );
 		memcpy( line_len_new, ss->line_len, sizeof(int) * ss->lines_alloc );
+		free( ss->line_len );
 		ss->lines_alloc = lines_alloc_new;
 		ss->line_len = line_len_new;
 	}
@@ -71,6 +130,7 @@ int stream_impl_pop_line( struct stream_impl_data *ss )
 	return len;
 }
 
+#ifdef DEBUG
 static void dump_contents( struct colm_program *prg, struct stream_impl_data *sid )
 {
 	struct run_buf *rb = sid->queue.head;
@@ -83,6 +143,7 @@ static void dump_contents( struct colm_program *prg, struct stream_impl_data *si
 		rb = rb->next;
 	}
 }
+#endif
 
 static bool loc_set( location_t *loc )
 {
@@ -91,9 +152,7 @@ static bool loc_set( location_t *loc )
 
 static void close_stream_file( FILE *file )
 {
-	if ( file != stdin && file != stdout && file != stderr &&
-			fileno(file) != 0 && fileno( file) != 1 && fileno(file) != 2 )
-	{
+	if ( file != stdin && file != stdout && file != stderr ) {
 		fclose( file );
 	}
 }
@@ -166,7 +225,7 @@ struct run_buf *new_run_buf( int sz )
 }
 
 /* Keep the position up to date after consuming text. */
-void update_position_data( struct stream_impl_data *is, const char *data, long length )
+void update_position_data( struct stream_impl_data *is, const alph_t *data, long length )
 {
 	int i;
 	for ( i = 0; i < length; i++ ) {
@@ -184,7 +243,7 @@ void update_position_data( struct stream_impl_data *is, const char *data, long l
 }
 
 /* Keep the position up to date after sending back text. */
-void undo_position_data( struct stream_impl_data *is, const char *data, long length )
+void undo_position_data( struct stream_impl_data *is, const alph_t *data, long length )
 {
 	/* FIXME: this needs to fetch the position information from the parsed
 	 * token and restore based on that.. */
@@ -207,7 +266,8 @@ void undo_position_data( struct stream_impl_data *is, const char *data, long len
  * Interface
  */
 
-static void data_transfer_loc( struct colm_program *prg, location_t *loc, struct stream_impl_data *ss )
+static void data_transfer_loc( struct colm_program *prg, location_t *loc,
+		struct stream_impl_data *ss )
 {
 	loc->name = ss->name;
 	loc->line = ss->line;
@@ -219,7 +279,8 @@ static void data_transfer_loc( struct colm_program *prg, location_t *loc, struct
  * Data inputs: files, strings, etc.
  */
 
-static int data_get_data( struct colm_program *prg, struct stream_impl_data *ss, char *dest, int length )
+static int data_get_data( struct colm_program *prg, struct stream_impl_data *ss,
+		alph_t *dest, int length )
 {
 	int copied = 0;
 
@@ -229,7 +290,8 @@ static int data_get_data( struct colm_program *prg, struct stream_impl_data *ss,
 		if ( buf == 0 ) {
 			/* Got through the in-mem buffers without copying anything. */
 			struct run_buf *run_buf = new_run_buf( 0 );
-			int received = ss->funcs->get_data_source( prg, (struct stream_impl*)ss, run_buf->data, FSM_BUFSIZE );
+			int received = ss->funcs->get_data_source( prg,
+					(struct stream_impl*)ss, run_buf->data, FSM_BUFSIZE );
 			if ( received == 0 ) {
 				free( run_buf );
 				break;
@@ -246,7 +308,7 @@ static int data_get_data( struct colm_program *prg, struct stream_impl_data *ss,
 		/* Anything available in the current buffer. */
 		if ( avail > 0 ) {
 			/* The source data from the current buffer. */
-			char *src = &buf->data[buf->offset];
+			alph_t *src = &buf->data[buf->offset];
 
 			int slen = avail < length ? avail : length;
 			memcpy( dest+copied, src, slen ) ;
@@ -276,7 +338,8 @@ static struct stream_impl *data_split_consumed( program_t *prg, struct stream_im
 	return split_off;
 }
 
-int data_append_data( struct colm_program *prg, struct stream_impl_data *sid, const char *data, int length )
+int data_append_data( struct colm_program *prg, struct stream_impl_data *sid,
+		const alph_t *data, int length )
 {
 	struct run_buf *tail = sid->queue.tail;
 	if ( tail == 0 || length > (FSM_BUFSIZE - tail->length) ) {
@@ -342,7 +405,7 @@ int data_undo_append_data( struct colm_program *prg, struct stream_impl_data *si
 
 static void data_destructor( program_t *prg, tree_t **sp, struct stream_impl_data *si )
 {
-	if ( si->file != 0 )
+	if ( si->file != 0 && !si->no_file_close )
 		close_stream_file( si->file );
 	
 	if ( si->collect != 0 ) {
@@ -368,6 +431,9 @@ static void data_destructor( program_t *prg, tree_t **sp, struct stream_impl_dat
 	// if ( si->name != 0 )
 	//	free( si->name );
 
+	if ( si->line_len != 0 )
+		free( si->line_len );
+
 	free( si );
 }
 
@@ -384,22 +450,33 @@ static void data_flush_stream( struct colm_program *prg, struct stream_impl_data
 
 static void data_close_stream( struct colm_program *prg, struct stream_impl_data *si )
 {
-	if ( si->file != 0 ) {
+	if ( si->file != 0 && !si->no_file_close )
 		close_stream_file( si->file );
-		si->file = 0;
-	}
+
+	si->file = 0;
+}
+
+static int data_get_option( struct colm_program *prg, struct stream_impl_data *si, int option )
+{
+	return si->auto_trim;
+}
+
+static void data_set_option( struct colm_program *prg, struct stream_impl_data *si, int option, int value )
+{
+	si->auto_trim = value ? 1 : 0;
 }
 
 static void data_print_tree( struct colm_program *prg, tree_t **sp,
 		struct stream_impl_data *si, tree_t *tree, int trim )
 {
 	if ( si->file != 0 )
-		colm_print_tree_file( prg, sp, (struct stream_impl*)si, tree, false );
+		colm_print_tree_file( prg, sp, si, tree, trim );
 	else if ( si->collect != 0 )
-		colm_print_tree_collect( prg, sp, si->collect, tree, false );
+		colm_print_tree_collect( prg, sp, si->collect, tree, trim );
 }
 
-static int data_get_parse_block( struct colm_program *prg, struct stream_impl_data *ss, int *pskip, char **pdp, int *copied )
+static int data_get_parse_block( struct colm_program *prg, struct stream_impl_data *ss,
+		int *pskip, alph_t **pdp, int *copied )
 {
 	int ret = 0;
 	*copied = 0;
@@ -410,7 +487,8 @@ static int data_get_parse_block( struct colm_program *prg, struct stream_impl_da
 		if ( buf == 0 ) {
 			/* Got through the in-mem buffers without copying anything. */
 			struct run_buf *run_buf = new_run_buf( 0 );
-			int received = ss->funcs->get_data_source( prg, (struct stream_impl*)ss, run_buf->data, FSM_BUFSIZE );
+			int received = ss->funcs->get_data_source( prg,
+					(struct stream_impl*)ss, run_buf->data, FSM_BUFSIZE );
 			if ( received == 0 ) {
 				free( run_buf );
 				ret = INPUT_EOD;
@@ -432,7 +510,7 @@ static int data_get_parse_block( struct colm_program *prg, struct stream_impl_da
 		/* Anything available in the current buffer. */
 		if ( avail > 0 ) {
 			/* The source data from the current buffer. */
-			char *src = &buf->data[buf->offset];
+			alph_t *src = &buf->data[buf->offset];
 
 			/* Need to skip? */
 			if ( *pskip > 0 && *pskip >= avail ) {
@@ -460,7 +538,8 @@ static int data_get_parse_block( struct colm_program *prg, struct stream_impl_da
 	return ret;
 }
 
-static int data_consume_data( struct colm_program *prg, struct stream_impl_data *sid, int length, location_t *loc )
+static int data_consume_data( struct colm_program *prg, struct stream_impl_data *sid,
+		int length, location_t *loc )
 {
 	int consumed = 0;
 	int remaining = length;
@@ -505,9 +584,10 @@ static int data_consume_data( struct colm_program *prg, struct stream_impl_data
 	return consumed;
 }
 
-static int data_undo_consume_data( struct colm_program *prg, struct stream_impl_data *sid, const char *data, int length )
+static int data_undo_consume_data( struct colm_program *prg, struct stream_impl_data *sid,
+		const alph_t *data, int length )
 {
-	const char *end = data + length;
+	const alph_t *end = data + length;
 	int amount = length;
 	if ( amount > sid->consumed )
 		amount = sid->consumed;
@@ -538,7 +618,8 @@ static int data_undo_consume_data( struct colm_program *prg, struct stream_impl_
 	}
 
 	debug( prg, REALM_INPUT, "data_undo_consume_data: stream %p "
-			"undid consume %d of %d bytes, consumed now %d, \n", sid, amount, length, sid->consumed );
+			"undid consume %d of %d bytes, consumed now %d, \n",
+			sid, amount, length, sid->consumed );
 
 #ifdef DEBUG
 	dump_contents( prg, sid );
@@ -551,7 +632,8 @@ static int data_undo_consume_data( struct colm_program *prg, struct stream_impl_
  * File Inputs
  */
 
-static int file_get_data_source( struct colm_program *prg, struct stream_impl_data *si, char *dest, int length )
+static int file_get_data_source( struct colm_program *prg, struct stream_impl_data *si,
+		alph_t *dest, int length )
 {
 	return fread( dest, 1, length, si->file );
 }
@@ -560,7 +642,8 @@ static int file_get_data_source( struct colm_program *prg, struct stream_impl_da
  * Text inputs
  */
 
-static int accum_get_data_source( struct colm_program *prg, struct stream_impl_data *si, char *dest, int want )
+static int accum_get_data_source( struct colm_program *prg, struct stream_impl_data *si,
+		alph_t *dest, int want )
 {
 	long avail = si->dlen - si->offset;
 	long take = avail < want ? avail : want;
@@ -594,10 +677,14 @@ struct stream_funcs_data file_funcs =
 	&data_flush_stream,
 	&data_close_stream,
 	&data_print_tree,
+
 	&data_split_consumed,
 	&data_append_data,
 	&data_undo_append_data,
 	&data_destructor,
+
+	&data_get_option,
+	&data_set_option,
 };
 
 struct stream_funcs_data accum_funcs = 
@@ -619,6 +706,9 @@ struct stream_funcs_data accum_funcs =
 	&data_append_data,
 	&data_undo_append_data,
 	&data_destructor,
+
+	&data_get_option,
+	&data_set_option,
 };
 
 static void si_data_init( struct stream_impl_data *is, char *name )
@@ -632,12 +722,14 @@ static void si_data_init( struct stream_impl_data *is, char *name )
 	is->byte = 0;
 
 	/* Indentation turned off. */
-	is->level = COLM_INDENT_OFF;
+	is->indent.level = COLM_INDENT_OFF;
+	is->indent.indent = 0;
 }
 
 struct stream_impl *colm_impl_new_accum( char *name )
 {
-	struct stream_impl_data *si = (struct stream_impl_data*)malloc(sizeof(struct stream_impl_data));
+	struct stream_impl_data *si = (struct stream_impl_data*)
+			malloc(sizeof(struct stream_impl_data));
 	si_data_init( si, name );
 	si->funcs = (struct stream_funcs*)&accum_funcs;
 
@@ -646,7 +738,8 @@ struct stream_impl *colm_impl_new_accum( char *name )
 
 static struct stream_impl *colm_impl_new_file( char *name, FILE *file )
 {
-	struct stream_impl_data *ss = (struct stream_impl_data*)malloc(sizeof(struct stream_impl_data));
+	struct stream_impl_data *ss = (struct stream_impl_data*)
+			malloc(sizeof(struct stream_impl_data));
 	si_data_init( ss, name );
 	ss->funcs = (struct stream_funcs*)&file_funcs;
 	ss->file = file;
@@ -655,16 +748,24 @@ static struct stream_impl *colm_impl_new_file( char *name, FILE *file )
 
 static struct stream_impl *colm_impl_new_fd( char *name, long fd )
 {
-	struct stream_impl_data *si = (struct stream_impl_data*)malloc(sizeof(struct stream_impl_data));
+	struct stream_impl_data *si = (struct stream_impl_data*)
+			malloc(sizeof(struct stream_impl_data));
 	si_data_init( si, name );
 	si->funcs = (struct stream_funcs*)&file_funcs;
-	si->file = fdopen( fd, ( fd == 0 ) ? "r" : "w" );
+
+	const char *mode = ( fd == 0 ) ? "r" : "w";
+	si->file = colm_fd_open( fd, mode );
+#ifndef HAVE_FOPENCOOKIE
+	if ( fd == 0 || fd == 1 || fd == 2 )
+		si->no_file_close = 1;
+#endif
 	return (struct stream_impl*)si;
 }
 
 struct stream_impl *colm_impl_consumed( char *name, int len )
 {
-	struct stream_impl_data *si = (struct stream_impl_data*)malloc(sizeof(struct stream_impl_data));
+	struct stream_impl_data *si = (struct stream_impl_data*)
+			malloc(sizeof(struct stream_impl_data));
 	si_data_init( si, name );
 	si->funcs = (struct stream_funcs*)&accum_funcs;
 
@@ -677,24 +778,32 @@ struct stream_impl *colm_impl_consumed( char *name, int len )
 	return (struct stream_impl*)si;
 }
 
-struct stream_impl *colm_impl_new_text( char *name, const char *data, int len )
+struct stream_impl *colm_impl_new_text( char *name, struct colm_location *loc, const alph_t *data, int len )
 {
-	struct stream_impl_data *si = (struct stream_impl_data*)malloc(sizeof(struct stream_impl_data));
+	struct stream_impl_data *si = (struct stream_impl_data*)
+			malloc(sizeof(struct stream_impl_data));
 	si_data_init( si, name );
 	si->funcs = (struct stream_funcs*)&accum_funcs;
 
-	char *buf = (char*)malloc( len );
+	alph_t *buf = (alph_t*)malloc( len );
 	memcpy( buf, data, len );
 
 	si->data = buf;
 	si->dlen = len;
 
+	if ( loc != 0 ) {
+		si->line = loc->line;
+		si->column = loc->column;
+		si->byte = loc->byte;
+	}
+
 	return (struct stream_impl*)si;
 }
 
 struct stream_impl *colm_impl_new_collect( char *name )
 {
-	struct stream_impl_data *ss = (struct stream_impl_data*)malloc(sizeof(struct stream_impl_data));
+	struct stream_impl_data *ss = (struct stream_impl_data*)
+			malloc(sizeof(struct stream_impl_data));
 	si_data_init( ss, name );
 	ss->funcs = (struct stream_funcs*)&accum_funcs;
 	ss->collect = (struct colm_str_collect*) malloc( sizeof( struct colm_str_collect ) );
@@ -743,7 +852,7 @@ stream_t *colm_stream_open_file( program_t *prg, tree_t *name, tree_t *mode )
 	}
 	
 	/* Need to make a C-string (null terminated). */
-	char *file_name = (char*)malloc(string_length(head_name)+1);
+	char *file_name = malloc(string_length(head_name)+1);
 	memcpy( file_name, string_data(head_name), string_length(head_name) );
 	file_name[string_length(head_name)] = 0;
 
