@@ -118,6 +118,30 @@ tree_t *colm_get_repeat_val( tree_t *tree )
 	return kid->tree;
 }
 
+tree_t *colm_get_left_repeat_next( tree_t *tree )
+{
+	kid_t *kid = tree->child;
+
+	if ( tree->flags & AF_LEFT_IGNORE )
+		kid = kid->next;
+	if ( tree->flags & AF_RIGHT_IGNORE )
+		kid = kid->next;
+
+	return kid->tree;
+}
+
+tree_t *colm_get_left_repeat_val( tree_t *tree )
+{
+	kid_t *kid = tree->child;
+
+	if ( tree->flags & AF_LEFT_IGNORE )
+		kid = kid->next;
+	if ( tree->flags & AF_RIGHT_IGNORE )
+		kid = kid->next;
+	
+	return kid->next->tree;
+}
+
 int colm_repeat_end( tree_t *tree )
 {
 	kid_t *kid = tree->child;
@@ -1015,6 +1039,28 @@ kid_t *tree_child( program_t *prg, const tree_t *tree )
 	return kid;
 }
 
+/* Find the first child of a tree. */
+kid_t *tree_child_maybe_ignore( program_t *prg, const tree_t *tree, int with_ignore )
+{
+	struct lang_el_info *lel_info = prg->rtd->lel_info;
+	kid_t *kid = tree->child;
+
+	if ( !with_ignore ) {
+		if ( tree->flags & AF_LEFT_IGNORE )
+			kid = kid->next;
+		if ( tree->flags & AF_RIGHT_IGNORE )
+			kid = kid->next;
+	}
+
+	/* Skip over attributes. */
+	long object_length = lel_info[tree->id].object_length;
+	long a;
+	for ( a = 0; a < object_length; a++ )
+		kid = kid->next;
+
+	return kid;
+}
+
 /* Detach at the first real child of a tree. */
 kid_t *tree_extract_child( program_t *prg, tree_t *tree )
 {
@@ -1107,6 +1153,17 @@ tree_t *get_rhs_el( program_t *prg, tree_t *lhs, long position )
 	return pos->tree;
 }
 
+void set_rhs_el( program_t *prg, tree_t *lhs, long position, tree_t *value )
+{
+	kid_t *pos = tree_child( prg, lhs );
+	while ( position > 0 ) {
+		pos = pos->next;
+		position -= 1;
+	}
+	pos->tree = value;
+}
+
+
 kid_t *get_rhs_el_kid( program_t *prg, tree_t *lhs, long position )
 {
 	kid_t *pos = tree_child( prg, lhs );
